﻿using DataDisplayBoard.Model;
using MaxCommunication.DataBase.SQLServer;
using Microsoft.Practices.Prism.Commands;
using PropertyChanged;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Windows;

namespace DataDisplayBoard.ViewModel
{
    [AddINotifyPropertyChangedInterface]
    public class TaskViewModel
    {
        public SQLServerExtension sqlServerExtension { get; set; } = new SQLServerExtension("server=.;uid=sa;pwd=123;database=PMS_KLDZ_6324");

        public ObservableCollection<MesTaskModel> TaskCollection { get; set; } = new ObservableCollection<MesTaskModel>();

        public MesTaskModel MesTaskModel { get; set; }

        public string[] MesTaskTypeArr { get; set; } = new string[] { "挂表任务", "摘表任务", "移库任务" };

        public string[] TaskStatusArr { get; set; } = new string[] { "任务下发", "任务生成", "任务异常" , "任务单点完成" ,"任务完成" , "挂表上报完成", "工单完成" , "工单上报完成", "未知" };

        public string MesTaskType { get; set; } = "挂表任务";

        public DelegateCommand ReflushCommand => new DelegateCommand(ReflushTask);

        public DelegateCommand DeleteCommand => new DelegateCommand(DeleteTask);

        public DelegateCommand ExcerptWatchCommand => new DelegateCommand(ExcerptWatchTask);

        public DelegateCommand UpdateCommand => new DelegateCommand(UpdateTask);

        public TaskViewModel()
        {
            ReflushTask();
        }

        public void ReflushTask()
        {
            DataTable TaskTable = new DataTable(); TaskCollection.Clear();
            if (MesTaskType == "摘表任务") { TaskTable = sqlServerExtension.QueryTable("PMS_Max_MesEWTask", "order by CallTime desc"); }
            else if (MesTaskType == "挂表任务") { TaskTable = sqlServerExtension.QueryTable("PMS_Max_MesPWTask", "order by CallTime desc"); }
            else if (MesTaskType == "移库任务") { TaskTable = sqlServerExtension.QueryTable("PMS_Max_MesMoveTask", "order by CallTime desc"); }
            if (TaskTable != null && TaskTable.Rows.Count != 0)
            {
                TaskCollection.Clear();
                foreach (DataRow item in TaskTable.Rows)
                {
                    TaskCollection.Add(new MesTaskModel(item));
                }
            }
        }

        public void DeleteTask()
        {
            string TableName = "", TaskIDArr = "";
            if (MesTaskType == "摘表任务") { TableName = "PMS_Max_MesEWTask"; }
            else if (MesTaskType == "挂表任务") { TableName = "PMS_Max_MesPWTask"; }
            else if (MesTaskType == "移库任务") { TableName = "PMS_Max_MesMoveTask"; }
            if (TableName == null || TableName.Length == 0) { return; }
            foreach (var item in TaskCollection)
            {
                if (item.IsSelected)
                {
                    if (TaskIDArr.Length == 0)
                    {
                        TaskIDArr = $"{item.TaskID}";
                    }
                    else
                    {
                        TaskIDArr = TaskIDArr + $",{item.TaskID}";
                    }

                }
            }
            if (TaskIDArr == null || TaskIDArr.Length == 0) { return; }
            sqlServerExtension.DeleteRecord(TableName, $"where ID in ({TaskIDArr})");
            sqlServerExtension.Disconnect(); ReflushTask();
        }

        public void UpdateTask()
        {
            MessageBoxResult messageBoxResult= MessageBox.Show("确定要更新任务状态吗?", "提示", MessageBoxButton.OKCancel);
            if (messageBoxResult == MessageBoxResult.OK)
            {
                string TableName = "";
                int taskstatus = 0;
                int.TryParse(MesTaskModel.TaskStatus, out taskstatus);
                if (MesTaskType == "摘表任务")
                {
                    TableName = "PMS_Max_MesEWTask";
                    sqlServerExtension.UpdateRecord(TableName, $"TaskStatus = {taskstatus}", $"where ID = {MesTaskModel.TaskID}");
                }
                else if (MesTaskType == "挂表任务")
                {
                    TableName = "PMS_Max_MesPWTask";
                    sqlServerExtension.UpdateRecord(TableName, $"TaskStatus = {taskstatus}", $"where ID = {MesTaskModel.TaskID}");
                }
                else if (MesTaskType == "移库任务")
                {
                    //TableName = "PMS_Max_MesMoveTask";
                    //sqlServerExtension.UpdateRecord(TableName, $"TaskStatus = {taskstatus}", $"where ID = {MesTaskModel.TaskID}");
                }
                sqlServerExtension.Disconnect();
                MessageBox.Show("操作成功!");
            }
            
        }
        string TaskNo; string DeceiveNo;
        public void ExcerptWatchTask()
        {
            SQLServerExtension sqlServerExtention = new SQLServerExtension("server =.; uid=sa;pwd=123;database=PMS_KLDZ_6324");
            if (DeceiveNo == null && DeceiveNo.Length == 0) { return; }
            DataTable dataTable = sqlServerExtension.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveName = '{DeceiveNo}'");
            if (dataTable == null || dataTable.Rows.Count == 0) { return; }
            dataTable = sqlServerExtension.QueryTable("PMS_Max_" + dataTable.Rows[0]["WorkStationName"]?.ToString(), $"where Status = 1 and Quantity<>0");//PMS_Max_MesEWTask
            if (dataTable == null || dataTable.Rows.Count == 0) { return; }

            //string values = $"0,'{TrayBarcodeModel.TrayBarcode}','{TrayBarcodeModel.Barcode1}','{TrayBarcodeModel.Barcode2}','{TrayBarcodeModel.Barcode3}'," +
            //             $"'{TrayBarcodeModel.Barcode4}','{TrayBarcodeModel.Barcode5}','{TrayBarcodeModel.Barcode6}','{TrayBarcodeModel.Barcode7}','{TrayBarcodeModel.Barcode8}'," +
            //             $"'{TrayBarcodeModel.Barcode9}','{TrayBarcodeModel.Barcode10}','{TrayBarcodeModel.Barcode11}','{TrayBarcodeModel.Barcode12}'";
            //sqlServerExtention.AddRecord("PMS_Max_MesEWTask", values);
        }
    }
}
