﻿using CasunCommon.SystemLog;
using HslCommunication;
using HslCommunication.Profinet.Siemens;
using Pms.UserEvent.Config;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using System.Threading;
using System.Threading.Tasks;

namespace Pms.UserEvent.Communication
{ 
    public class AbnormalCommunication
    {
        public static string ExceptionName { get; set; } = "WareHouse_100";
        public static string CT_EnterName { get; set; } = "CT_Enter";

        public TaskService taskService { get; set; } = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);

        public DeceiveService deceiveService { get; set; } = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);

        public WareHouseService wareHouseService { get; set; } = new WareHouseService(CConfigManager.gs_globalConfig.WMSDataBaseUrl);

        public SiemensS7Net siemensS7Net { get; set; }

        private bool outMaterialRequest { get; set; }

        public bool OutMaterialRequest
        {
            get { return outMaterialRequest; }
            set
            {
                if (!outMaterialRequest && value)
                {
                    outMaterialRequest = value;
                    //WareHouseModel wareHouseModel = wareHouseService.GetWareHouseModel(ExceptionName); if (wareHouseModel == null) { return; }
                    //wareHouseModel.TopLayerNumber = wareHouseModel.MaterialType == EnumType.MaterialTypeEnum.SinglePhaseMeter ? 12 : 4;
                    //wareHouseModel.LayerNumber = GetTrayLayer(); wareHouseService.SetFullWareHouseModel(wareHouseModel);
                    //taskService.AddPWMesTask(wareHouseModel.TaskNo, wareHouseModel.PileNo, ExceptionName, (int)wareHouseModel.MaterialType);
                }
                outMaterialRequest = value;
            }
        }

        public AbnormalCommunication(string Ip, int Port)
        {
            siemensS7Net = new SiemensS7Net(SiemensPLCS.S1200, Ip); siemensS7Net.Port = Port;
            Task.Factory.StartNew(ReceiveOutMaterialRequest);
        }

        public void ReceiveOutMaterialRequest()
        {
            while (true)
            {
                OutMaterialRequest = ReadBool("DB1.0.3");
                Thread.Sleep(1000);
            }
        }

        public bool ReadBool(string Address)
        {
            lock (siemensS7Net)
            {
                OperateResult<bool> operateResult = siemensS7Net.ReadBool(Address);
                if (operateResult.IsSuccess)
                {
                    return operateResult.Content;
                }
                else
                {
                    CLog.Instance.WriteLog("AbnormalAlarmLog", "中控读取异常位PLC数据失败！");
                    return false;
                }
            }
        }

        public bool WriteBool(string Address, bool Value)
        {
            lock (siemensS7Net)
            {
                OperateResult operateResult = siemensS7Net.Write(Address, Value);
                return operateResult.IsSuccess;
            }
        }

        public int GetTrayLayer()
        {
            if (ReadBool("DB1.1.5"))
            {
                return 6;
            }
            else if (ReadBool("DB1.1.4"))
            {
                return 5;
            }
            else if (ReadBool("DB1.1.3"))
            {
                return 4;
            }
            else if (ReadBool("DB1.1.2"))
            {
                return 3;
            }
            else if (ReadBool("DB1.1.1"))
            {
                return 2;
            }
            else if (ReadBool("DB1.1.0"))
            {
                return 1;
            }
            else
            {
                return 0;
            }
        }
    }
}
