﻿using CasunCommon.SystemLog;
using Pms.Common.SystemEnum;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.UserEvent.Communication;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using Pms.UserEvent.SystemEvent;
using System;
using System.Threading;

namespace Pms.UserEvent.OrderEvent
{
    public partial class COrderEvent:BaseOrderEvent
    {
        public COrderEvent():base()
        {}

        /// <summary>
        /// 任务创建前触发此事件
        /// 设置 e.IsSuccessed = false 则任务不再执行 即任务异常
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderBeginCreate(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// 任务下发AGVS后触发
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderStarted(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// AGVS回调 小车给AGV后触发
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderAssignedCarrier(object sender, P_OrderEventArgs e)
        {
        }


        // 当前版本不会触发
        protected override void OrderExecuting(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// 任务异常不能执行时，会触发此事件
        /// 此处添加可以 自动处理的异常 的处理代码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderException(object sender, P_OrderEventArgs e)
        {
            CarService carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService.UpdateEmptyCarRoll(e.OrderData.Carrier);

            TaskService taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            if (e.OrderData.BookTemplateID_RW.Contains("TaskBook_PocketWatch"))
            {
                taskService.UpdatePWTaskStatus(e.OrderData.Reserved1, (int)PocketWatchEnum.TaskException);
            }
            else if (e.OrderData.BookTemplateID_RW.Contains("TaskBook_Excerpt"))
            {
                taskService.UpdateEWTaskStatus(e.OrderData.Reserved1, (int)ExcerptWatchEnum.TaskException);
            }
        }


        /// <summary>
        /// 调用取消接口后触发此事件
        /// 任务取消成功与否 都会触发此事件
        /// e.IsCancelSuccess = True ： 取消成功
        /// e.IsCancelSuccess = false： 取消失败
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderCanceled(object sender, P_OrderEventArgs e)
        {
        }

        // 当前版本不会触发
        protected override void OrderChanged(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// 任务执行完成会触发此事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderFinished(object sender, P_OrderEventArgs e)
        {
            if (e.OrderData.BookTemplateID_RW == "TaskBook_General_1" || e.OrderData.BookTemplateID_RW == "TaskBook_General_2")
            {
                TaskService taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                if (e.OrderData.Reserved3 == "PWTask")//1
                {
                    taskService.UpdatePWTaskStatus(e.OrderData.Reserved2, (int)PocketWatchEnum.TaskFinished);
                }
                else if (e.OrderData.Reserved3 == "MWTask")//1
                {
                    taskService.UpdateMWTaskStatus(e.OrderData.Reserved2, (int)PocketWatchEnum.TaskFinished);
                }
            }
        }

        /// <summary>
        /// 如果配置了步骤行为60035 则需要处理此事件
        /// 用来决定任务AGV小车是否可以 释放事件  
        /// e.IsSuccessed = true; 则 系统认为可以释放小车
        /// e.IsSuccessed = false; 则 系统认为不可以 释放小车
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderCarrierNeedRelease(object sender, P_OrderEventArgs e)
        {
            e.IsSuccessed = true;
        }

        /// <summary>
        /// 任务模板 选择时触发此事件
        /// e.OrderData.BookTemplateID_RW = 值
        /// 设置 e.IsSuccessed = false 则任务不再执行 即任务异常
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderTemplateSelecting(object sender, P_OrderEventArgs e)
        {

        }

        /// <summary>
        /// 系统重启时处理历史任务时触发
        /// 设置 e.ExecuteCode 值选择不同的处理方式
        /// e.ExecuteCode 值
        /// 0 ：默认处理 方法 已开始的任务置为异常；未开始的任务 ，如果隔天直接删除 ，未隔天 则重新开始
        /// 1 ：置为异常
        /// 2 ：重新开始 
        /// 3 ：直接删除
        /// 其余值 默认处理方法
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void RestartHisOrderExecute(object sender, P_RestartHisOrderExecuteEventArgs e)
        {
            e.ExecuteCode = 3;
        }
    }
}
