﻿using CasunCommon.SystemLog;
using MaxCommunication.BaseType.WebApi;
using MaxCommunication.DataBase.SQLServer;
using Newtonsoft.Json;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.Model;
using Pms.UserEvent.SystemEvent;
using System;
using System.Collections.Generic;
using System.Data;

namespace Pms.UserEvent.Service
{
    public class MesService
    {
        public SQLServerExtension sqlServerExtension_Pms { get; set; }

        public WebApiClient webApiClient = new WebApiClient();


        public MesService(string ConnectString)
        {
            sqlServerExtension_Pms = new SQLServerExtension(ConnectString);
        }
        public void PickUpFinished(string PileNo, string StationNo)
        {
            try
            {
                CLog.Instance.WriteLog("AGV取货货完成上传", string.Format("请求内容:{0},开始请求时间:{1}", JsonConvert.SerializeObject(new ReportMesModel(PileNo, StationNo)), DateTime.Now));
                string result = webApiClient.SendData_Post(CConfigManager.gs_globalConfig.MesInterface_5_3, JsonConvert.SerializeObject(new ReportMesModel(PileNo, StationNo)), null);
                CLog.Instance.WriteLog("AGV取货货完成上传", string.Format("响应内容:{0},响应时间:{1}", result != null ? result : "", DateTime.Now));
                MesResultModel mesResultModel = JsonConvert.DeserializeObject<MesResultModel>(result);
                if (mesResultModel != null && mesResultModel.RESULT_FLAG == "1")
                {

                }
            }
            catch (Exception ex)
            {
                CLog.Instance.WriteLog("MesErrorApi", "Mes Api Interface_5_3：" + ex.ToString());
            }
        }
        /// <summary>
        /// AGV送货到达上传
        /// </summary>
        /// <param name="PileNo"></param>
        /// <param name="StationNo"></param>
        public void DeliveryFinished(string PileNo, string StationNo)
        {
            try
            {
                CLog.Instance.WriteLog("AGV送货到达上传", string.Format("请求内容:{0},开始请求时间:{1}", JsonConvert.SerializeObject(new ReportMesModel(PileNo, StationNo)), DateTime.Now));
                string result = webApiClient.SendData_Post(CConfigManager.gs_globalConfig.MesInterface_5_4, JsonConvert.SerializeObject(new ReportMesModel(PileNo, StationNo)), null);
                CLog.Instance.WriteLog("AGV送货到达上传", string.Format("响应内容:{0},响应时间:{1}", result != null ? result : "", DateTime.Now));
                MesResultModel mesResultModel = JsonConvert.DeserializeObject<MesResultModel>(result);
                if (mesResultModel != null && mesResultModel.RESULT_FLAG == "1")
                {

                }
            }
            catch (Exception ex)
            {
                CLog.Instance.WriteLog("MesErrorApi", "Mes Api Interface_5_4：" + ex.ToString());
            }
        }

        public bool ReportTrayAndBarcode(BindTrayAndBarcodeModel bindTrayAndBarcodeModel)
        {
            try
            {
                CLog.Instance.WriteLog("箱表校验上传", string.Format("请求内容:{0},开始请求时间:{1}", JsonConvert.SerializeObject(bindTrayAndBarcodeModel), DateTime.Now));
                string result = webApiClient.SendData_Post(CConfigManager.gs_globalConfig.MesInterface_5_5, JsonConvert.SerializeObject(bindTrayAndBarcodeModel), null);
                CLog.Instance.WriteLog("箱表校验上传", string.Format("响应内容:{0},响应时间:{1}", result != null ? result : "", DateTime.Now));
                MesResultModel mesResultModel = JsonConvert.DeserializeObject<MesResultModel>(result);
                if (mesResultModel != null && mesResultModel.RESULT_FLAG == "1")
                {
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                CLog.Instance.WriteLog("MesErrorApi", "Mes Api Interface_5_5：" + ex.ToString());
                return false;
            }
        }

        public bool ReportPocketWatchFinish(PocketWatchFinishModel pocketWatchFinishModel)
        {
            try
            {
                CLog.Instance.WriteLog("挂表完成上传", string.Format("请求内容:{0},开始请求时间:{1}", JsonConvert.SerializeObject(pocketWatchFinishModel), DateTime.Now));
                string result = webApiClient.SendData_Post(CConfigManager.gs_globalConfig.MesInterface_5_6, JsonConvert.SerializeObject(pocketWatchFinishModel), null);
                CLog.Instance.WriteLog("挂表完成上传", string.Format("响应内容:{0},响应时间:{1}", result != null ? result : "", DateTime.Now));
                MesResultModel mesResultModel = JsonConvert.DeserializeObject<MesResultModel>(result);
                if (mesResultModel != null && mesResultModel.RESULT_FLAG == "1")
                {
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                CLog.Instance.WriteLog("MesErrorApi", "Mes Api Interface_5_6：" + ex.ToString());
                return false;
            }
        }
        /// <summary>
        /// 上报工单(摘表)完成后组箱组垛关系或挂表完成后剩余箱表关系
        /// </summary>
        /// <param name="excerptWatchFinishModel"></param>
        /// <returns></returns>
        public bool ReportExcerptWatchFinish(ExcerptWatchFinishModel excerptWatchFinishModel)
        {
            try
            {
                CLog.Instance.WriteLog("箱表关系上传", string.Format("请求内容:{0},开始请求时间:{1}", JsonConvert.SerializeObject(excerptWatchFinishModel), DateTime.Now));
                string result = webApiClient.SendData_Post(CConfigManager.gs_globalConfig.MesInterface_5_8, JsonConvert.SerializeObject(excerptWatchFinishModel), null);
                CLog.Instance.WriteLog("箱表关系上传", string.Format("响应内容:{0},响应时间:{1}", result != null ? result : "", DateTime.Now));
                MesResultModel mesResultModel = JsonConvert.DeserializeObject<MesResultModel>(result);
                if (mesResultModel != null && mesResultModel.RESULT_FLAG == "1")
                {
                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                CLog.Instance.WriteLog("MesErrorApi", "Mes Api Interface_5_8：" + ex.ToString());
                return false;
            }
        }
        public List<string> GetDeceiveList(int MaterialType)
        {

            try
            {
                List<string> DeceiveList = new List<string>();
                DataTable dt_Deceive = sqlServerExtension_Pms.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveType = {MaterialType}");
                sqlServerExtension_Pms.Disconnect();
                if (dt_Deceive == null || dt_Deceive.Rows.Count == 0) { return null; }
                foreach (DataRow item_Deceive in dt_Deceive.Rows)
                {
                    DeceiveList.Add(item_Deceive["DeceiveName"]?.ToString());

                }
                return DeceiveList;
            }
            catch
            {
                return null;
            }
        }
        public string GetStationState(int MaterialType, List<string> DeceiveListCorrect, out List<string> DeceiveList)
        {
            DeceiveList = new List<string>();
            try
            {
                DataTable dt_Deceive = sqlServerExtension_Pms.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveType = {MaterialType}");
                sqlServerExtension_Pms.Disconnect();
                if (dt_Deceive == null || dt_Deceive.Rows.Count == 0) { return null; }

                //DeceiveList.Add("1");
                foreach (DataRow item_Deceive in dt_Deceive.Rows)
                {
                    DeceiveList.Add(item_Deceive["DeceiveName"]?.ToString());
                    if (!CSystemEvent.AbutmentEnable)
                    {
                        return item_Deceive["DeceiveName"]?.ToString();
                    }
                }
                if (MaterialType == 6)
                {//互感器
                    return dt_Deceive.Rows[0]["DeceiveName"]?.ToString();
                }
                if (DeceiveListCorrect != null)
                {
                    DeceiveList.RemoveAll(p => !DeceiveListCorrect.Contains(p));
                }
                DeceiveList.Reverse();
                int count = DeceiveList.Count;
                if (count > 2)
                {
                    DeceiveList.RemoveAt(2);
                }
                CLog.Instance.WriteLog("询问台体", string.Format("请求内容:{0},开始请求时间:{1}", JsonConvert.SerializeObject(DeceiveList), DateTime.Now));
                string result = webApiClient.SendData_Post(CConfigManager.gs_globalConfig.MesInterface_5_9, JsonConvert.SerializeObject(DeceiveList), null);//询问站台信息
                CLog.Instance.WriteLog("询问台体", string.Format("响应内容:{0},响应时间:{1}", result != null ? result : "", DateTime.Now));
                MesResultModel mesResultModel = JsonConvert.DeserializeObject<MesResultModel>(result);
                if (mesResultModel == null || mesResultModel.RESULT_FLAG != "1" || mesResultModel.STATION_STATE.Count == 0) { return null; }
                foreach (var item in mesResultModel.STATION_STATE)
                {
                    if (item.STATE == "1")
                    {
                        if (JudgementDeceiveIsHasTask(item.STATION_ID))
                        {
                            return item.STATION_ID;
                        }
                    }
                }
                return null;
            }
            catch (Exception ex)
            {
                CLog.Instance.WriteLog("MesErrorApi", "Mes Api Interface_5_9：" + ex.ToString());
                return null;
            }
        }
        //20220609 add
        public bool JudgementDeceiveIsHasTask(string DeceiveName)
        {
            string tempStr = ToPmsName(DeceiveName);
            CPmsControllers.Instance.OrderDataController.GetOrderList($"OrderState <> 255 and PosArr like '%{tempStr}%'", "", out List<OrderData> OrderList, out EFailCode failCode);
            if (OrderList != null && OrderList.Count != 0)
            {
                return false;
            }
            return true;
        }

        public string ToPmsName(string MesName)
        {
            DataTable dataTable = sqlServerExtension_Pms.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveName = '{MesName}'");
            sqlServerExtension_Pms.Disconnect();
            if (dataTable != null && dataTable.Rows.Count != 0)
            {
                return dataTable.Rows[0]["WorkStationName"]?.ToString();
            }
            return null;
        }
    }
}
