﻿using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Communication;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using Pms.UserEvent.SystemEvent;
using System;
using System.Collections.Generic;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 续接
    /// </summary>
    public class Behavior_65023 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public WareHouseService wareHouseService { get; set; }

        public WorkOrderService workOrderService { get; set; }

        public MesService mesService { get; set; }

        public CacheService cacheService { get; set; }

        public Behavior_65023() : base(65023)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            wareHouseService = new WareHouseService(CConfigManager.gs_globalConfig.WMSDataBaseUrl);
            workOrderService = new WorkOrderService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            mesService = new MesService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            cacheService = new CacheService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(orderData.Carrier);
            if (orderData.BookTemplateID == "TaskBook_PocketWatchF"|| orderData.BookTemplateID == "TaskBook_PocketWatchB_F")
            {
                if (carModel.FirstRollLayer != 0)
                {
                    string DeceiveName = orderData.PosArr.Length == 3 ? orderData.PosArr[2] : orderData.PosArr.Length == 2 ? orderData.PosArr[1] : orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2);
                    int MaterialType = deceiveService.GetMaterialType(DeceiveName);
                    int TaskPoint = deceiveService.JudgementDeceiveIsPocketWatch(DeceiveName, MaterialType);//orderData.PosArr[1]
                    if (TaskPoint != 0)
                    {
                        if (orderData.BookTemplateID != "TaskBook_PocketWatchB_F" && carModel.FirstTopLayerNumber < (carModel.FirstRollMaterialType == 1 ? 12 : 4))
                        {
                            //string DeceiveName = orderData.PosArr.Length == 3 ? orderData.PosArr[2] : orderData.PosArr.Length == 2 ? orderData.PosArr[1] : orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2);
                            taskService.PocketWatchBehindPreTask(DeceiveName + "_" + TaskPoint, orderData.OrderID,
                           CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                        }
                        else//orderData.PosArr[1]
                        {
                            taskService.PocketWatchBehindTask(DeceiveName + "_" + TaskPoint, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition, MaterialType, orderData.Reserved2);
                        }
                    }
                    else
                    {
                        string EmptyWareHouse = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (EmptyWareHouse == null || EmptyWareHouse.Length == 0) { e.IsSuccessed = false; return; }
                        taskService.UninstallTask(EmptyWareHouse, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        taskService.UpdatePWTaskStatus(orderData.Reserved1, (int)PocketWatchEnum.TaskException);
                    }
                }
            }
            else if (orderData.BookTemplateID == "TaskBook_PocketWatchSB" || orderData.BookTemplateID == "TaskBook_PocketWatchTB")
            {
                int MaterialType = deceiveService.GetMaterialType(orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2));
                MesPWTaskModel mesTaskModel = taskService.GetPWMesTask(orderData.Reserved1);
                switch ((PocketWatchEnum)Convert.ToInt32(mesTaskModel.TaskStatus))
                {
                    case PocketWatchEnum.TaskException://卸表到移栽机
                        if(CSystemEvent.abnormalCommunication.OutMaterialRequest)//异常位有物料,等待
                        {
                            e.IsSuccessed = false; return;
                        }
                        string ExceptionWareHouse = wareHouseService.SearchExceptionWareHouse();
                        if (ExceptionWareHouse != null && ExceptionWareHouse.Length > 0)//异常位未被占用
                        {
                            if (carModel.SecondRollLayer > 0)
                            {
                                string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length == 0)
                                {
                                    wareHouseService.UpdateWareHouseStatus(ExceptionWareHouse, 2);
                                    e.IsSuccessed = false; return;
                                }
                                taskService.UninstallDoubleTask(ExceptionWareHouse, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                            }
                            else
                            {
                                taskService.UninstallTask(AbnormalCommunication.ExceptionName, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                            }
                        }
                        else
                        {
                            e.IsSuccessed = false; return;
                        }
                        break;
                    case PocketWatchEnum.TaskPointFinished:
                    case PocketWatchEnum.TaskCreated:
                        string DeceiveName = orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2);
                        if (carModel.FirstRollLayer != 0)
                        {
                            int TaskPoint = deceiveService.JudgementDeceiveIsPocketWatch(orderData.PosArr[0], MaterialType);
                            if (DeceiveName + "_" + TaskPoint == orderData.PosArr[0])//20210802
                            {//跳步才是正常逻辑
                                e.IsDefaultNext = false;
                                break;
                            }
                            if (TaskPoint != 0)
                            {
                                taskService.PocketWatchBehindTask(DeceiveName + "_" + TaskPoint, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition, MaterialType, orderData.Reserved2);
                                taskService.UpdatePWTaskStatus(orderData.Reserved1, (int)PocketWatchEnum.TaskCreated);
                            }
                            else
                            {
                                string EmptyTrayPoint2_1 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint2_1 == null || EmptyTrayPoint2_1.Length == 0) { e.IsSuccessed = false; return; }
                                if (carModel.SecondRollLayer != 0)
                                {
                                    string EmptyTrayPoint2_2 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                    if (EmptyTrayPoint2_2 == null || EmptyTrayPoint2_2.Length == 0) { wareHouseService.UpdateWareHouseStatus(EmptyTrayPoint2_1, 2); e.IsSuccessed = false; return; }
                                    taskService.UninstallDoubleTask(EmptyTrayPoint2_1, EmptyTrayPoint2_2, orderData.OrderID, orderData.Reserved1);
                                }
                                else
                                {
                                    taskService.UninstallTask(EmptyTrayPoint2_1, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                                }
                                taskService.UpdatePWTaskStatus(orderData.Reserved1, 10);
                            }
                        }
                        else
                        {
                            string EmptyTrayPoint2_3 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint2_3 == null || EmptyTrayPoint2_3.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallTask(EmptyTrayPoint2_3, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                            taskService.UpdatePWTaskStatus(orderData.Reserved1, 10);
                        }
                        break;
                    case PocketWatchEnum.PocketWatchFinished:
                        string EmptyTrayPoint3 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (EmptyTrayPoint3 == null || EmptyTrayPoint3.Length == 0) { e.IsSuccessed = false; return; }
                        if (carModel.FirstRollLayer > 0 && carModel.SecondRollLayer > 0)
                        {
                            string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length == 0)
                            {
                                wareHouseService.UpdateWareHouseStatus(EmptyTrayPoint3, 2);
                                e.IsSuccessed = false; return;
                            }
                            taskService.UninstallDoubleTask(EmptyTrayPoint3, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                        }
                        else if (carModel.FirstRollLayer > 0)
                        {
                            taskService.UninstallTask(EmptyTrayPoint3, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                        else if (carModel.SecondRollLayer > 0)
                        {
                            taskService.UninstallTask(EmptyTrayPoint3, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        }
                        break;
                    case PocketWatchEnum.TaskFinished:
                        //先上报挂表完成
                        MesService mesService0 = new MesService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                        PocketWatchFinishModel pocketWatchFinishModel0 = new PocketWatchFinishModel(deceiveService.ToMesName(e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 2, 2)));
                        List<DeceiveModel> DeceiveList0 = deceiveService.GetDeceiveModel(e.TaskBook.WareHouse);
                        if (DeceiveList0 == null || DeceiveList0.Count == 0) { e.IsSuccessed = false; return; }
                        //List<DeceiveModel> DeceiveEmptyList = DeceiveList.FindAll(t => t.Status == 2);
                        //if (DeceiveEmptyList != null && DeceiveEmptyList.Count != 0) { e.IsSuccessed = true; return; }
                        foreach (var item in DeceiveList0)
                        {
                            if (item.Status == 1)
                            {
                                pocketWatchFinishModel0.METER.Add(new RelationshipModel(item.EpitopeNumber, item.Barcode));
                            }
                        }
                        if (mesService0.ReportPocketWatchFinish(pocketWatchFinishModel0))
                        {
                            taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.PocketWatchFinished));
                        }
                        e.IsSuccessed = false; return;
                        //string EmptyTrayPoint3 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        //if (EmptyTrayPoint3 == null || EmptyTrayPoint3.Length == 0) { e.IsSuccessed = false; return; }
                        //if(carModel.FirstRollLayer>0&&carModel.SecondRollLayer>0)
                        //{
                        //    string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        //    if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length == 0) 
                        //    {
                        //        wareHouseService.UpdateWareHouseStatus(EmptyTrayPoint3, 2);
                        //        e.IsSuccessed = false; return;
                        //    }
                        //    taskService.UninstallDoubleTask(EmptyTrayPoint3, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                        //}
                        //else if(carModel.FirstRollLayer > 0 )
                        //{
                        //    taskService.UninstallTask(EmptyTrayPoint3, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        //}
                        //else if ( carModel.SecondRollLayer > 0)
                        //{
                        //    taskService.UninstallTask(EmptyTrayPoint3, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        //}

                        break;
                    case PocketWatchEnum.PocketWatchWorkOrderFinished:
                        if (carModel.FirstRollLayer != 0)
                        {
                            string warehouse7 = wareHouseService.SearchWareHouse7();
                            if (warehouse7 == null || warehouse7.Length == 0)
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                                if (carModel.SecondRollLayer != 0)
                                {
                                    string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                    if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length == 0) { wareHouseService.UpdateWareHouseStatus(EmptyTrayPoint4, 2); e.IsSuccessed = false; return; }
                                    taskService.UninstallDoubleTask(EmptyTrayPoint4, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);//多出表处理
                                }
                                else
                                {
                                    taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                                }
                            }
                            else
                            {
                                if (carModel.SecondRollLayer != 0)
                                {
                                    string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                    if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { wareHouseService.UpdateWareHouseStatus(warehouse7, 2); e.IsSuccessed = false; return; }
                                    taskService.UninstallDoubleTask("WareHouse_7", EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);//多出表处理
                                }
                                else
                                {
                                    taskService.UninstallTask("WareHouse_7", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                                }
                            }
                        }
                        else
                        {
                            if (carModel.SecondRollLayer != 0)
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                                taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                            }

                        }
                        break;
                    case PocketWatchEnum.WorkOrderFinished:
                        //先上报挂表完成
                        MesService mesService = new MesService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                        PocketWatchFinishModel pocketWatchFinishModel = new PocketWatchFinishModel(deceiveService.ToMesName(e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 2, 2)));
                        List<DeceiveModel> DeceiveList = deceiveService.GetDeceiveModel(e.TaskBook.WareHouse);
                        if (DeceiveList == null || DeceiveList.Count == 0) { e.IsSuccessed = false; return; }
                        //List<DeceiveModel> DeceiveEmptyList = DeceiveList.FindAll(t => t.Status == 2);
                        //if (DeceiveEmptyList != null && DeceiveEmptyList.Count != 0) { e.IsSuccessed = true; return; }
                        foreach (var item in DeceiveList)
                        {
                            if (item.Status == 1)
                            {
                                pocketWatchFinishModel.METER.Add(new RelationshipModel(item.EpitopeNumber, item.Barcode));
                            }
                        }
                        bool reported= mesService.ReportPocketWatchFinish(pocketWatchFinishModel);
                        //再上报剩余箱表绑定关系，再拉到回料点
                        CarService carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                        TrayBarcodeService trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                        //OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
                        //CarModel carModel100 = carService.GetCar(100); if (carModel100 == null) { }
                        //if (carModel.FirstRollLayer == 6 && carModel.FirstTopLayerNumber == 4)
                        {
                            ExcerptWatchFinishModel excerptWatchFinishModel = new ExcerptWatchFinishModel("", mesTaskModel.TaskNo);
                            excerptWatchFinishModel.PILE_TYPE = "0" + mesTaskModel.MaterialType;
                            List<TrayBarcodeModel> TrayBarcodeList = trayBarcodeService.GetAllTrayBarcodeRemain(carModel.FirstPileNo);//trayBarcodeService.GetAllTrayBarcodeRemain(100);
                            if (TrayBarcodeList != null)
                            {
                                foreach (var item in TrayBarcodeList)
                                {
                                    TrayRelationshipModel trayRelationshipModel = new TrayRelationshipModel(item.TrayBarcode);
                                    if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                                    if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                                    if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                                    if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                                    if (item.Barcode5 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("5", item.Barcode5)); }
                                    if (item.Barcode6 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("6", item.Barcode6)); }
                                    if (item.Barcode7 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("7", item.Barcode7)); }
                                    if (item.Barcode8 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("8", item.Barcode8)); }
                                    if (item.Barcode9 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("9", item.Barcode9)); }
                                    if (item.Barcode10 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("10", item.Barcode10)); }
                                    if (item.Barcode11 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("11", item.Barcode11)); }
                                    if (item.Barcode12 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("12", item.Barcode12)); }
                                    if (trayRelationshipModel.METER.Count > 0)
                                    {
                                        if (string.IsNullOrEmpty(excerptWatchFinishModel.PILE_NO))
                                        {
                                            excerptWatchFinishModel.PILE_NO = item.TrayBarcode;
                                        }
                                        else
                                        {
                                            excerptWatchFinishModel.PILE_NO = excerptWatchFinishModel.PILE_NO + "_" + item.TrayBarcode;
                                        }

                                        excerptWatchFinishModel.BOX.Add(trayRelationshipModel);
                                    }

                                }
                                if (!string.IsNullOrEmpty(excerptWatchFinishModel.PILE_NO))
                                {
                                    mesService.ReportExcerptWatchFinish(excerptWatchFinishModel);
                                }
                            }
                        }
                        if(reported)
                        {
                            taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.PocketWatchWorkOrderFinished));
                        }
                        e.IsSuccessed = false; return;
                        //if (carModel.FirstRollLayer != 0)
                        //{
                        //    string warehouse7= wareHouseService.SearchWareHouse7();
                        //    if (warehouse7 == null || warehouse7.Length == 0)
                        //    {
                        //        string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        //        if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                        //        if (carModel.SecondRollLayer != 0)
                        //        {
                        //            string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        //            if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length == 0) { wareHouseService.UpdateWareHouseStatus(EmptyTrayPoint4, 2); e.IsSuccessed = false; return; }
                        //            taskService.UninstallDoubleTask(EmptyTrayPoint4, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);//多出表处理
                        //        }
                        //        else
                        //        {
                        //            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        //        }
                        //    }
                        //    else
                        //    {
                        //        if (carModel.SecondRollLayer != 0)
                        //        {
                        //            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        //            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { wareHouseService.UpdateWareHouseStatus(warehouse7, 2); e.IsSuccessed = false; return; }
                        //            taskService.UninstallDoubleTask("WareHouse_7", EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);//多出表处理
                        //        }
                        //        else
                        //        {
                        //            taskService.UninstallTask("WareHouse_7", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        //        }
                        //    }
                        //}
                        //else
                        //{
                        //    if (carModel.SecondRollLayer != 0)
                        //    {
                        //        string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        //        if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                        //        taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        //    }

                        //}
                        break;
                }
            }
            else if (orderData.BookTemplateID == "TaskBook_ExcerptWatchF" || orderData.BookTemplateID == "TaskBook_ExcerptWatchF1" || orderData.BookTemplateID == "TaskBook_ExcerptWatchF2"|| orderData.BookTemplateID == "TaskBook_ExcerptWatchB_F")
            {
                string DeceiveName = orderData.PosArr.Length == 3 ? orderData.PosArr[2] : orderData.PosArr.Length == 2 ? orderData.PosArr[1] : orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2);
                int MaterialType = deceiveService.GetMaterialType(DeceiveName);
                int TaskPoint = deceiveService.JudgementDeceiveIsExcerptWatch(DeceiveName, MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2);
                if (carModel.FirstRollLayer != 0 && MaterialType == 1 && carModel.FirstTopLayerNumber != 12 && orderData.BookTemplateID != "TaskBook_ExcerptWatchB_F" )//考虑把不满行单单相表抓到AGV缓存台
                {
                    if (TaskPoint != 0)
                    {
                        taskService.ExcerptWatchBehindPreTask(DeceiveName + "_" + TaskPoint, orderData.OrderID,
                            CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                    }
                    else
                    {
                        string UninstallPoint = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (UninstallPoint == null || UninstallPoint.Length == 0) { e.IsSuccessed = false; return; }
                        taskService.UninstallTask(UninstallPoint, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskException);
                    }
                }
                else if (carModel.FirstRollLayer != 0 && MaterialType != 1 && carModel.FirstTopLayerNumber != 4 && orderData.BookTemplateID != "TaskBook_ExcerptWatchB_F")
                {
                    if (TaskPoint != 0)
                    {
                        taskService.ExcerptWatchBehindPreTask(DeceiveName + "_" + TaskPoint, orderData.OrderID,
                            CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                    }
                    else
                    {
                        string UninstallPoint = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (UninstallPoint == null || UninstallPoint.Length == 0) { e.IsSuccessed = false; return; }
                        taskService.UninstallTask(UninstallPoint, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskException);
                    }
                }
                else //if (carModel.SecondRollLayer != 0)
                {
                    //string DeceiveName = orderData.PosArr.Length == 3 ? orderData.PosArr[2] : orderData.PosArr[1];
                    //int MaterialType = deceiveService.GetMaterialType(DeceiveName);
                    //int TaskPoint = deceiveService.JudgementDeceiveIsExcerptWatch(DeceiveName, MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2);
                    if (TaskPoint != 0)
                    {
                        taskService.ExcerptWatchBehindTask(DeceiveName + "_" + TaskPoint, orderData.OrderID,
                            CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                    }
                    else
                    {
                        string UninstallPoint = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (UninstallPoint == null || UninstallPoint.Length == 0) { e.IsSuccessed = false; return; }
                        taskService.UninstallTask(UninstallPoint, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskException);
                    }
                }
            }
            else if (orderData.BookTemplateID.Contains("TaskBook_ExcerptWatchSB"))
            {
                int MaterialType = deceiveService.GetMaterialType(orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2));
                int TaskPoint = deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2);
                string DeceiveName = orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2);
                //bool IsNG = Convert.ToInt32(orderData.Reserved3) == 2;
                if (carModel.FirstRollLayer == 6 && carModel.FirstTopLayerNumber == 12&& orderData.BookTemplateID.Contains("OK"))
                {
                    //if (TaskPoint != 0) { taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskIssued); }
                    if (carModel.SecondRollLayer != 0)
                    {
                        string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                        {
                            e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                            return;
                        }
                        taskService.UninstallDoubleTask("CT_Enter", EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                    }
                    else
                    {
                        taskService.UninstallTask("CT_Enter", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                    }
                    taskService.UpdateEWTaskStatus(orderData.Reserved1, 10);
                    if (deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2) != 0)
                    {
                        taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                    }
                }
                else if (carModel.FirstRollLayer == 6 && carModel.FirstTopLayerNumber == 12 && orderData.BookTemplateID.Contains("NG"))
                {
                    //if (TaskPoint != 0) { taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskIssued); }
                    string WareHouse7 = wareHouseService.SearchWareHouse7();
                    if (carModel.SecondRollLayer != 0)
                    {
                        if (WareHouse7 == null || WareHouse7.Length < 1)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                            {
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length < 1)
                            {
                                wareHouseService.SetEmptyWareHouseModel(EmptyTrayPoint4);
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallDoubleTask(EmptyTrayPoint4, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                        }
                        else
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                            {
                                wareHouseService.SetEmptyWareHouseModel(WareHouse7);
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallDoubleTask(WareHouse7, EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                        }
                    }
                    else
                    {
                        if (WareHouse7 == null || WareHouse7.Length < 1)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                            {
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                        else
                        {
                            taskService.UninstallTask(WareHouse7, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                    }

                    taskService.UpdateEWTaskStatus(orderData.Reserved1, 10);
                    if (deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2) != 0)
                    {
                        taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                    }
                }
                //else if (carModel.SecondRollLayer == 0 && carModel.SecondTopLayerNumber == 0 && e.StepBook.StepID== 60)
                //{//2022-5-23新增:2号滚筒只有一个托盘，并且托盘上没有表则执行默认下一步。
                //    e.IsSuccessed = true;
                //    return;
                //}
                else if (workOrderService.JudegementWorkOrderBFinshed(carModel.TaskNo) && orderData.BookTemplateID.Contains("OK"))//20220607 新增
                {
                    TrayBarcodeService trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                    ExcerptWatchFinishModel excerptWatchFinishModel = new ExcerptWatchFinishModel(carModel.FirstPileNo, orderData.Reserved2);
                    List<TrayBarcodeModel> TrayBarcodeList = trayBarcodeService.GetAllTrayBarcode(carModel.FirstPileNo);
                    foreach (var item in TrayBarcodeList)
                    {
                        TrayRelationshipModel trayRelationshipModel = new TrayRelationshipModel(item.TrayBarcode);
                        if (carModel.FirstRollMaterialType == 1)
                        {
                            if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                            if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                            if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                            if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                            if (item.Barcode5 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("5", item.Barcode5)); }
                            if (item.Barcode6 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("6", item.Barcode6)); }
                            if (item.Barcode7 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("7", item.Barcode7)); }
                            if (item.Barcode8 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("8", item.Barcode8)); }
                            if (item.Barcode9 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("9", item.Barcode9)); }
                            if (item.Barcode10 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("10", item.Barcode10)); }
                            if (item.Barcode11 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("11", item.Barcode11)); }
                            if (item.Barcode12 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("12", item.Barcode12)); }
                        }
                        else
                        {
                            if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                            if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                            if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                            if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                        }
                        excerptWatchFinishModel.BOX.Add(trayRelationshipModel);
                    }
                    mesService.ReportExcerptWatchFinish(excerptWatchFinishModel);
                    //taskService.UninstallTask("CT_Enter", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);

                    
                    if (carModel.FirstRollLayer != 0)
                    {
                        if (carModel.SecondRollLayer != 0)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallDoubleTask("CT_Enter", EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                        }
                        else
                        {
                            taskService.UninstallTask("CT_Enter", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                    }
                    else
                    {
                        if (carModel.SecondRollLayer != 0)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        }
                    }
                }
                else if (workOrderService.JudegementWorkOrderBFinshed(carModel.TaskNo) && orderData.BookTemplateID.Contains("NG"))//20220607 新增
                {
                    TrayBarcodeService trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                    ExcerptWatchFinishModel excerptWatchFinishModel = new ExcerptWatchFinishModel(carModel.FirstPileNo, orderData.Reserved2);
                    List<TrayBarcodeModel> TrayBarcodeList = trayBarcodeService.GetAllTrayBarcode(carModel.FirstPileNo);
                    foreach (var item in TrayBarcodeList)
                    {
                        TrayRelationshipModel trayRelationshipModel = new TrayRelationshipModel(item.TrayBarcode);
                        if (carModel.FirstRollMaterialType == 1)
                        {
                            if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                            if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                            if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                            if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                            if (item.Barcode5 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("5", item.Barcode5)); }
                            if (item.Barcode6 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("6", item.Barcode6)); }
                            if (item.Barcode7 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("7", item.Barcode7)); }
                            if (item.Barcode8 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("8", item.Barcode8)); }
                            if (item.Barcode9 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("9", item.Barcode9)); }
                            if (item.Barcode10 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("10", item.Barcode10)); }
                            if (item.Barcode11 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("11", item.Barcode11)); }
                            if (item.Barcode12 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("12", item.Barcode12)); }
                        }
                        else
                        {
                            if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                            if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                            if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                            if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                        }
                        excerptWatchFinishModel.BOX.Add(trayRelationshipModel);
                    }
                    mesService.ReportExcerptWatchFinish(excerptWatchFinishModel);
                    //string WareHouse7 = wareHouseService.SearchWareHouse7();
                    //string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                    //if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                    if (carModel.FirstRollLayer != 0)
                    {
                        string WareHouse7 = wareHouseService.SearchWareHouse7();
                        if (carModel.SecondRollLayer != 0)
                        {
                            if (WareHouse7 == null || WareHouse7.Length < 1)
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                                {
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length < 1)
                                {
                                    wareHouseService.SetEmptyWareHouseModel(EmptyTrayPoint4);
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                taskService.UninstallDoubleTask(EmptyTrayPoint4, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                            }
                            else
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                                {
                                    wareHouseService.SetEmptyWareHouseModel(WareHouse7);
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                taskService.UninstallDoubleTask(WareHouse7, EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                            }
                        }
                        else
                        {
                            if (WareHouse7 == null || WareHouse7.Length < 1)
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                                {
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                            }
                            else
                            {
                                taskService.UninstallTask(WareHouse7, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                            }
                        }
                    }
                    else
                    {
                        if (carModel.SecondRollLayer != 0)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        }
                    }
                }
                else
                {
                    if (TaskPoint != 0)
                    {
                        //bool hasExcerptWatch = orderData.BookTemplateID.Contains("OK") ? deceiveService.HasExcerptWatchOK(e.TaskBook.WareHouse, orderData.Reserved2) : deceiveService.HasExcerptWatchNG(e.TaskBook.WareHouse, orderData.Reserved2);
                        if (carModel.SecondRollLayer == 0 && carModel.FirstTopLayerNumber == 12)//2滚筒没有空托盘了 20220616
                        {
                            //20220621 add 直接续接取空箱摘表任务
                            if (carModel.FirstRollLayer != 6)
                            {
                                string EmptyTrayWareHouse = wareHouseService.SearchEmptyTray(MaterialType);
                                if (EmptyTrayWareHouse != null && EmptyTrayWareHouse.Length != 0)
                                {

                                    taskService.ExcerptWatchFrontTask3(EmptyTrayWareHouse, DeceiveName, orderData.OrderID, CSystemEvent.SecondRollerPosition, Convert.ToInt32(orderData.Reserved1), orderData.Reserved2, Convert.ToInt32(orderData.Reserved3));
                                    taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskCreated);
                                    e.IsDefaultNext = true; //未找到缓存台空库位，继续寻找
                                    e.IsSuccessed = true;
                                    return;
                                }
                            }

                            string EmptyWareHouse1 = workOrderService.JudegementWorkOrderBFinshed(orderData.Reserved2) && orderData.BookTemplateID.Contains("OK") ? "CT_Enter" : wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyWareHouse1 == null || EmptyWareHouse1.Length == 0)
                            {
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallTask(EmptyWareHouse1, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition, orderData.Reserved2, orderData.Reserved3);
                            //taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskIssued);
                            taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));//继续添加摘表任务
                        }
                        else
                        {
                            if (DeceiveName + "_" + TaskPoint == orderData.PosArr[0])//20210802
                            {//跳步才是正常逻辑
                                e.IsDefaultNext = false;
                            }
                            else
                            {
                                taskService.ExcerptWatchBehindTask(DeceiveName + "_" + TaskPoint, orderData.OrderID, CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                                taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskCreated);
                            }
                        }
                    }
                    else
                    {
                        if (cacheService.SearchFullStore(e.TaskBook.Carrier, orderData.Reserved2, Convert.ToInt32(orderData.Reserved3), out int RowIndex, out string[] Barcode)
                            && (carModel.FirstTopLayerNumber != 12 || carModel.SecondRollLayer != 0))
                        {//跳步才是正常逻辑
                            e.IsDefaultNext = false;//20210802
                            //taskService.ExcerptWatchBehindTask(e.TaskBook.WareHouse, orderData.OrderID, CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                            //taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskCreated);
                        }
                        else
                        {
                            //20220621 add

                            string EmptyWareHouse1 = workOrderService.JudegementWorkOrderBFinshed(orderData.Reserved2) && orderData.BookTemplateID.Contains("OK") ? "CT_Enter" : wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyWareHouse1 == null || EmptyWareHouse1.Length == 0) { e.IsSuccessed = false; return; }
                            if (carModel.SecondRollLayer != 0)
                            {
                                string EmptyWareHouse2 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyWareHouse2 == null || EmptyWareHouse2.Length == 0) { wareHouseService.UpdateWareHouseStatus(EmptyWareHouse1, 2); e.IsSuccessed = false; return; }
                                taskService.UninstallDoubleTask(EmptyWareHouse1, EmptyWareHouse2, orderData.OrderID, orderData.Reserved1, orderData.Reserved2, orderData.Reserved3);
                            }
                            else
                            {
                                taskService.UninstallTask(EmptyWareHouse1, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition, orderData.Reserved2, orderData.Reserved3);
                            }
                            taskService.UpdateEWTaskStatus(orderData.Reserved1, 10);
                        }
                    }
                }
            }
            else if (orderData.BookTemplateID.Contains("TaskBook_ExcerptWatchTB"))
            {
                int MaterialType = deceiveService.GetMaterialType(orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2));
                int TaskPoint = deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2);
                string DeceiveName = orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 2, 2);
                bool IsNG = Convert.ToInt32(orderData.Reserved3) == 2;
                if ((carModel.FirstRollLayer == 6 && carModel.FirstTopLayerNumber == 4) && orderData.BookTemplateID.Contains("OK"))
                {
                    if (carModel.SecondRollLayer != 0)
                    {
                        string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                        {
                            e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                            return;
                        }
                        taskService.UninstallDoubleTask("CT_Enter", EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                    }
                    else
                    {
                        taskService.UninstallTask("CT_Enter", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                    }
                    taskService.UpdateEWTaskStatus(orderData.Reserved1, 10);
                    int tempTaskType = Convert.ToInt32(orderData.Reserved3);
                    if (tempTaskType == 1)
                    {
                        if (deceiveService.HasExcerptWatchOK(e.TaskBook.WareHouse, orderData.Reserved2))//deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2) != 0
                        {
                            taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                        }
                    }
                    else if (tempTaskType == 2)
                    {
                        if (deceiveService.HasExcerptWatchNG(e.TaskBook.WareHouse, orderData.Reserved2))//deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2) != 0
                        {
                            taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                        }
                    }

                }
                else if ((carModel.FirstRollLayer == 6 && carModel.FirstTopLayerNumber == 4) && orderData.BookTemplateID.Contains("NG"))
                {
                    string WareHouse7 = wareHouseService.SearchWareHouse7();
                    if (carModel.SecondRollLayer != 0)
                    {
                        if (WareHouse7 == null || WareHouse7.Length < 1)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                            {
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length < 1)
                            {
                                wareHouseService.SetEmptyWareHouseModel(EmptyTrayPoint4);
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallDoubleTask(EmptyTrayPoint4, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                        }
                        else
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                            {
                                wareHouseService.SetEmptyWareHouseModel(WareHouse7);
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallDoubleTask(WareHouse7, EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                        }
                    }
                    else
                    {
                        if (WareHouse7 == null || WareHouse7.Length < 1)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                            {
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                        else
                        {
                            taskService.UninstallTask(WareHouse7, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                    }
                    taskService.UpdateEWTaskStatus(orderData.Reserved1, 10);
                    int tempTaskType = Convert.ToInt32(orderData.Reserved3);
                    if (tempTaskType == 1)
                    {
                        if (deceiveService.HasExcerptWatchOK(e.TaskBook.WareHouse, orderData.Reserved2))//deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2) != 0
                        {
                            taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                        }
                    }
                    else if (tempTaskType == 2)
                    {
                        if (deceiveService.HasExcerptWatchNG(e.TaskBook.WareHouse, orderData.Reserved2))//deceiveService.JudgementDeceiveIsExcerptWatch(orderData.PosArr[0], MaterialType, Convert.ToInt32(orderData.Reserved3), orderData.Reserved2) != 0
                        {
                            taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                        }
                    }

                }
                else if (workOrderService.JudegementWorkOrderBFinshed(carModel.TaskNo) && orderData.BookTemplateID.Contains("OK"))//20220327 新增
                {
                    TrayBarcodeService trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                    ExcerptWatchFinishModel excerptWatchFinishModel = new ExcerptWatchFinishModel(carModel.FirstPileNo, orderData.Reserved2);
                    List<TrayBarcodeModel> TrayBarcodeList = trayBarcodeService.GetAllTrayBarcode(carModel.FirstPileNo);
                    foreach (var item in TrayBarcodeList)
                    {
                        TrayRelationshipModel trayRelationshipModel = new TrayRelationshipModel(item.TrayBarcode);
                        if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                        if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                        if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                        if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                        excerptWatchFinishModel.BOX.Add(trayRelationshipModel);
                    }
                    mesService.ReportExcerptWatchFinish(excerptWatchFinishModel);
                    //taskService.UninstallTask("CT_Enter", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                    if (carModel.FirstRollLayer != 0)
                    {
                        if (carModel.SecondRollLayer != 0)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallDoubleTask("CT_Enter", EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                        }
                        else
                        {
                            taskService.UninstallTask("CT_Enter", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                        }
                    }
                    else
                    {
                        if (carModel.SecondRollLayer != 0)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        }
                    }
                }
                else if (workOrderService.JudegementWorkOrderBFinshed(carModel.TaskNo) && orderData.BookTemplateID.Contains("NG"))//20220327 新增
                {
                    TrayBarcodeService trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
                    ExcerptWatchFinishModel excerptWatchFinishModel = new ExcerptWatchFinishModel(carModel.FirstPileNo, orderData.Reserved2);
                    List<TrayBarcodeModel> TrayBarcodeList = trayBarcodeService.GetAllTrayBarcode(carModel.FirstPileNo);
                    foreach (var item in TrayBarcodeList)
                    {
                        TrayRelationshipModel trayRelationshipModel = new TrayRelationshipModel(item.TrayBarcode);
                        if (item.Barcode1 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("1", item.Barcode1)); }
                        if (item.Barcode2 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("2", item.Barcode2)); }
                        if (item.Barcode3 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("3", item.Barcode3)); }
                        if (item.Barcode4 != "0") { trayRelationshipModel.METER.Add(new RelationshipModel("4", item.Barcode4)); }
                        excerptWatchFinishModel.BOX.Add(trayRelationshipModel);
                    }
                    mesService.ReportExcerptWatchFinish(excerptWatchFinishModel);
                    //taskService.UninstallTask("WareHouse_7", orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                    if (carModel.FirstRollLayer != 0)
                    {
                        string WareHouse7 = wareHouseService.SearchWareHouse7();
                        if (carModel.SecondRollLayer != 0)
                        {
                            if (WareHouse7 == null || WareHouse7.Length < 1)
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                                {
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                string EmptyTrayPoint5 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint5 == null || EmptyTrayPoint5.Length < 1)
                                {
                                    wareHouseService.SetEmptyWareHouseModel(EmptyTrayPoint4);
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                taskService.UninstallDoubleTask(EmptyTrayPoint4, EmptyTrayPoint5, orderData.OrderID, orderData.Reserved1);
                            }
                            else
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                                {
                                    wareHouseService.SetEmptyWareHouseModel(WareHouse7);
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                taskService.UninstallDoubleTask(WareHouse7, EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1);
                            }
                        }
                        else
                        {
                            if (WareHouse7 == null || WareHouse7.Length < 1)
                            {
                                string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                                if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length < 1)
                                {
                                    e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                    return;
                                }
                                taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                            }
                            else
                            {
                                taskService.UninstallTask(WareHouse7, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                            }
                        }
                    }
                    else
                    {
                        if (carModel.SecondRollLayer != 0)
                        {
                            string EmptyTrayPoint4 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyTrayPoint4 == null || EmptyTrayPoint4.Length == 0) { e.IsSuccessed = false; return; }
                            taskService.UninstallTask(EmptyTrayPoint4, orderData.OrderID, orderData.Reserved1, CSystemEvent.SecondRollerPosition);
                        }
                    }
                }
                else
                {
                    if (TaskPoint != 0)
                    {
                        if (carModel.SecondRollLayer == 0 && carModel.FirstTopLayerNumber == 4)//2滚筒没有空托盘了 20220616
                        {
                            //20220621 add 直接续接取空箱摘表任务
                            if (carModel.FirstRollLayer != 6)
                            {
                                string EmptyTrayWareHouse = wareHouseService.SearchEmptyTray(MaterialType);
                                if (EmptyTrayWareHouse != null && EmptyTrayWareHouse.Length != 0)
                                {

                                    taskService.ExcerptWatchFrontTask3(EmptyTrayWareHouse, DeceiveName, orderData.OrderID, CSystemEvent.SecondRollerPosition, Convert.ToInt32(orderData.Reserved1), orderData.Reserved2, Convert.ToInt32(orderData.Reserved3));
                                    taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskCreated);
                                    e.IsDefaultNext = true; //未找到缓存台空库位，继续寻找
                                    e.IsSuccessed = true;
                                    return;
                                }
                            }

                            string EmptyWareHouse1 = workOrderService.JudegementWorkOrderBFinshed(orderData.Reserved2) && orderData.BookTemplateID.Contains("OK") ? "CT_Enter" : wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (EmptyWareHouse1 == null || EmptyWareHouse1.Length == 0)
                            {
                                e.IsSuccessed = false; //未找到缓存台空库位，继续寻找
                                return;
                            }
                            taskService.UninstallTask(EmptyWareHouse1, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition);
                            taskService.AddEWMesTask(orderData.Reserved2, deceiveService.ToMesName(DeceiveName), MaterialType, Convert.ToInt32(orderData.Reserved3));
                        }
                        else
                        {
                            if (DeceiveName + "_" + TaskPoint == orderData.PosArr[0])//20210802
                            {//跳步才是正常逻辑
                                e.IsDefaultNext = false;
                            }
                            else
                            {
                                taskService.ExcerptWatchBehindTask(DeceiveName + "_" + TaskPoint, orderData.OrderID, CSystemEvent.SecondRollerPosition, orderData.Reserved1, orderData.Reserved2, MaterialType, Convert.ToInt32(orderData.Reserved3));
                                taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskCreated);
                            }
                        }
                    }
                    else
                    {
                        string UninstallPoint1 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                        if (UninstallPoint1 == null || UninstallPoint1.Length == 0) { e.IsSuccessed = false; return; }
                        if (carModel.SecondRollLayer != 0)
                        {
                            string UninstallPoint2 = wareHouseService.SearchEmptyWareHouse(MaterialType);
                            if (UninstallPoint2 == null || UninstallPoint2.Length == 0) { wareHouseService.UpdateWareHouseStatus(UninstallPoint1, 2); e.IsSuccessed = false; return; }
                            taskService.UninstallDoubleTask(UninstallPoint1, UninstallPoint2, orderData.OrderID, orderData.Reserved1, orderData.Reserved2, orderData.Reserved3);
                        }
                        else
                        {
                            taskService.UninstallTask(UninstallPoint1, orderData.OrderID, orderData.Reserved1, CSystemEvent.FirstRollerPosition, orderData.Reserved2,orderData.Reserved3);
                        }
                        taskService.UpdateEWTaskStatus(orderData.Reserved1, 10);
                    }
                }
            }
            else if (orderData.BookTemplateID == "TaskBook_Uninstall1")
            {
                if (orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 4, 4) == AbnormalCommunication.ExceptionName)
                {
                    //WareHouseModel wareHouseModel = wareHouseService.GetWareHouseModel(AbnormalCommunication.ExceptionName);
                    //if (workOrderService.JudegementWorkOrderBFinshed(wareHouseModel.TaskNo))
                    //{
                    //    taskService.GeneralTask(orderData.PosArr[0], "Storehouse_Enter", CSystemEvent.FirstRollerPosition, "PWTask", Convert.ToInt32(orderData.Reserved1));
                    //}
                    //else
                    //{
                    //    string DeceiveName = mesService.GetStationState((int)wareHouseModel.MaterialType, out List<string> DeceiveList);
                    //    if (DeceiveName != null && DeceiveName.Length != 0 && deceiveService.JudgementDeceiveIsHasTask(DeceiveName))
                    //    {
                    //        int TaskPoint = deceiveService.JudgementDeceiveIsPocketWatch(deceiveService.ToPmsName(DeceiveName), (int)wareHouseModel.MaterialType);
                    //        if (TaskPoint != 0)
                    //        {
                    //            string TaskNo = taskService.GetPWMesTask(orderData.Reserved1).TaskNo;
                    //            taskService.PocketWatchFrontTask(orderData.PosArr[0], deceiveService.ToPmsName(DeceiveName), CSystemEvent.FirstRollerPosition, Convert.ToInt32(orderData.Reserved1), TaskNo);
                    //            taskService.UpdatePWTaskStatus(orderData.Reserved1, (int)PocketWatchEnum.TaskCreated);
                    //        }
                    //        else
                    //        {
                    //            string EmptyWareHouse = wareHouseService.SearchEmptyWareHouse((int)wareHouseModel.MaterialType);
                    //            if (EmptyWareHouse == null || EmptyWareHouse.Length == 0) { e.IsSuccessed = false; return; }
                    //            taskService.GeneralTask(orderData.PosArr[0], EmptyWareHouse, CSystemEvent.FirstRollerPosition, "PWTask", Convert.ToInt32(orderData.Reserved1));
                    //        }
                    //    }
                    //    else
                    //    {
                    //        string EmptyWareHouse = wareHouseService.SearchEmptyWareHouse((int)wareHouseModel.MaterialType);
                    //        if (EmptyWareHouse == null || EmptyWareHouse.Length == 0) { e.IsSuccessed = false; return; }
                    //        taskService.GeneralTask(orderData.PosArr[0], EmptyWareHouse, CSystemEvent.FirstRollerPosition, "PWTask", Convert.ToInt32(orderData.Reserved1));
                    //    }
                    //}
                }
                else if (orderData.PosArr[0].Remove(orderData.PosArr[0].Length - 4, 4) == AbnormalCommunication.CT_EnterName)
                {

                }
            }
            e.IsSuccessed = true;//异常：任务结束
        }
    }
}
