﻿using CasunCommon.SystemLog;
using MaxCommunication.DataBase.SQLServer;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.UserEvent.Communication;
using Pms.UserEvent.Config;
using Pms.UserEvent.SystemEvent;
using System;
using System.Data;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 请求PLC接料
    /// </summary>
    public class Behavior_65004 : BaseStepBookEvent
    {
        public Behavior_65004() : base(65004)
        {

        }
        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            string warehouse = e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 4, 4);
            if(CSystemEvent.DictWareHouseAbutmentInfo.ContainsKey(warehouse))
            {
                CSystemEvent.DictWareHouseAbutmentInfo[warehouse].Carrier = e.TaskBook.Carrier;
                CSystemEvent.DictWareHouseAbutmentInfo[warehouse].SignalClean = false;
            }
        }
        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
            {
            if (e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 4, 4) == AbnormalCommunication.ExceptionName)
            {
                e.IsSuccessed = ExceptionProtocol(); return;
            }
            else if (e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 4, 4) == WareHouseCommunication.WareHouse_In)
            {
                e.IsSuccessed = WareHouseProtocol(); return;
            }
            else
            {
                e.IsSuccessed = TransportProtocol(e.TaskBook.WareHouse); return;
            }
        }

        public bool TransportProtocol(string WareHouseID)
        {
            SQLServerExtension sqlServerExtension = new SQLServerExtension(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            DataTable dt = sqlServerExtension.QueryTable("PMS_Max_DeceiveManager", $"where FirstRoll = '{WareHouseID}' or SecondRoll = '{WareHouseID}'");
            sqlServerExtension.Disconnect();

            if (dt == null || dt.Rows.Count == 0)
            {
                CLog.Instance.WriteLog("DeceiveLog", "设备管理中未配置此输送线的地址偏移量，库位：" + WareHouseID);
                return false;
            }
            string Address = "DB10." + (Convert.ToInt32(dt.Rows[0]["Offset"]?.ToString())).ToString();
            if (!CSystemEvent.AbutmentEnable)
            {
                return true;
            }
            if (!CSystemEvent.transportCommunication.WriteBool(Address + ".0", true))
            {
                CLog.Instance.WriteLog("TransPortAlarmLog", "中控发送请求PLC接料命令失败，PLC地址：" + Address + ".0");
                return false;
            }
            if (!CSystemEvent.transportCommunication.ReadBool(Address + ".1"))
            {
                return false;
            }
            return true;
        }

        public bool ExceptionProtocol()
        {
            if (!CSystemEvent.abnormalCommunication.WriteBool("DB1.0.0", true))
            {
                CLog.Instance.WriteLog("AbnormalAlarmLog", "中控发送请求PLC接料命令失败，PLC地址：DB1.0.0");
                return false;
            }
            if (!CSystemEvent.abnormalCommunication.ReadBool("DB1.0.1"))
            {
                return false;
            }
            return true;
        }

        public bool WareHouseProtocol() 
        {
            if (!CSystemEvent.wareHouseCommunication.WriteBool("DB1304.0.0", true))//DB1.0.0
            {
                CLog.Instance.WriteLog("WareHouseAlarmLog", "中控发送请求PLC接料命令失败，PLC地址：DB1304.0.0");
                return false;
            }
            if (!CSystemEvent.wareHouseCommunication.ReadBool("DB1304.2.0"))//DB1.0.1
            {
                return false;
            }
            return true;
        }
    }
}
