﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using Pms.UserEvent.SystemEvent;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手抓空托盘前参数准备，可能跳步
    /// </summary>
    public class Behavior_65050 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public Behavior_65050() : base(65050)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            if((e.StepBook.StepID == 86 || e.StepBook.StepID == 87)&& CSystemEvent.SingleWatchHoldingJaw == 3)
            {
                CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
                if ((carModel.FirstTopLayerNumber == 12 || (carModel.FirstRollLayer == 0 && carModel.FirstTopLayerNumber == 0)) && carModel.SecondRollLayer != 0)
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { carModel.SecondRollLayer, 2, carModel.FirstRollLayer + 1 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65050，逻辑值：{carModel.SecondRollLayer}-2-{carModel.FirstRollLayer + 1}");

                    carModel.SecondRollLayer = carModel.SecondRollLayer - 1;
                    carModel.FirstRollLayer = carModel.FirstRollLayer + 1; carModel.FirstTopLayerNumber = 0;
                    carService.UpdateFullCarRoll(carModel);
                }
                else
                {
                    e.IsDefaultNext = false;
                }
            }
            else if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskFinished, ExcerptWatchEnum.TaskFinished }))
            {
                int tempAllowExcerptCount = CSystemEvent.SingleWatchHoldingJaw == 3 ? 3 : 1;
                CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
                if ((carModel.FirstTopLayerNumber == 12 || (carModel.FirstRollLayer == 0 && carModel.FirstTopLayerNumber == 0)) && carModel.SecondRollLayer != 0
                    && deceiveService.CanSingleExcerptWatchOK(e.TaskBook.WareHouse, e.TaskBook.Reserved2, out string[] Barcode, out int Index, out int ColumnIndex, out int ExcerptCount,tempAllowExcerptCount))
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { carModel.SecondRollLayer, 2, carModel.FirstRollLayer + 1 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65050，逻辑值：{carModel.SecondRollLayer}-2-{carModel.FirstRollLayer + 1}");

                    carModel.SecondRollLayer = carModel.SecondRollLayer - 1;
                    carModel.FirstRollLayer = carModel.FirstRollLayer + 1; carModel.FirstTopLayerNumber = 0;
                    carService.UpdateFullCarRoll(carModel);
                }
                else
                {
                    e.IsDefaultNext = false;
                }
            }
            else
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65050，逻辑值：0-0-0");
            }
            e.IsSuccessed = true;
        }
    }
}
