﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using Pms.UserEvent.SystemEvent;
using System;
using System.Linq;
using System.Threading;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手放表到托盘前参数准备
    /// </summary>
    public class Behavior_65053 : BaseStepBookEvent
    {
        public CarService carService { get; set; }

        public TaskService taskService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65053() : base(65053)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
            EventLibrary.Models.P_StepBook prev_Opeartor = e.TaskBook.StepBookDic.Values.FirstOrDefault(p => p.NextStepID == e.TaskBook.StepID);
            EventLibrary.Models.P_StepBook prev_Behavior = e.TaskBook.StepBookDic.Values.FirstOrDefault(p => p.NextStepID == prev_Opeartor.StepID);
            if (CSystemEvent.SingleWatchHoldingJaw == 3)
            {
                if (carModel.PullNumber != 7 && carModel.PullNumber != 0 && prev_Behavior.Behavior == 65059)//20220607 add
                {
                    int RowIndex = Convert.ToInt32(Math.Pow(2, Convert.ToInt32(carModel.FirstTopLayerNumber / 3)));
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, carModel.PullNumber, carModel.FirstRollLayer }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：{RowIndex}-{carModel.PullNumber}-{carModel.FirstRollLayer}");
                    string[] barcodes = carModel.PullNumber == 1 ? new string[] { carModel.Barcode1 } : new string[] { carModel.Barcode1, carModel.Barcode2 };
                    trayBarcodeService.UpdateTrayBarcode(barcodes, carModel.FirstTopLayerNumber + 1, e.TaskBook.Carrier);
                    carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber + carModel.PullNumber == 1 ? 1 : 2; carModel.PullNumber = 0;
                    carModel.Barcode1 = "0"; carModel.Barcode2 = "0"; carModel.Barcode3 = "0";
                    carService.UpdateFullCarRoll(carModel);
                }
                else if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished }))
                {
                    if (carModel.FirstTopLayerNumber != 12 && carModel.PullNumber == 7)
                    {
                        int RowIndex = Convert.ToInt32(Math.Pow(2, Convert.ToInt32(carModel.FirstTopLayerNumber / 3)));
                        CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, carModel.PullNumber, carModel.FirstRollLayer }, out int failcode1);
                        CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：{RowIndex}-{carModel.PullNumber}-{carModel.FirstRollLayer}");

                        trayBarcodeService.UpdateTrayBarcode(new string[] { carModel.Barcode1, carModel.Barcode2, carModel.Barcode3 }, carModel.FirstTopLayerNumber + 1, e.TaskBook.Carrier);
                        carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber + 3; carModel.PullNumber = 0;
                        carModel.Barcode1 = "0"; carModel.Barcode2 = "0"; carModel.Barcode3 = "0";
                        carService.UpdateFullCarRoll(carModel);
                    }
                    else
                    {
                        CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                        CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：0-0-0");
                    }
                }
                else
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：0-0-0");
                }
            }
            else
            {
                if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished }))
                {
                    if (carModel.FirstTopLayerNumber != 12 && carModel.PullNumber == 1)
                    {
                        int RowIndex = Convert.ToInt32(Math.Pow(2, Convert.ToInt32(carModel.FirstTopLayerNumber / 3)));
                        int temp = carModel.FirstTopLayerNumber % 3;
                        int ColumnIndex = temp == 0 ? 1 :temp==1?2:4;
                        CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, carModel.FirstRollLayer }, out int failcode1);
                        CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：{RowIndex}-{ColumnIndex}-{carModel.FirstRollLayer}");

                        trayBarcodeService.UpdateTrayBarcode(new string[] { carModel.Barcode1}, carModel.FirstTopLayerNumber + 1, e.TaskBook.Carrier);
                        carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber + 1; carModel.PullNumber = 0;
                        carModel.Barcode1 = "0"; carModel.Barcode2 = "0"; carModel.Barcode3 = "0";
                        carService.UpdateFullCarRoll(carModel);
                    }
                    else
                    {
                        CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                        CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：0-0-0");
                    }
                }
                else
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65053，逻辑值：0-0-0");
                }
            }
            e.IsSuccessed = true;
        }
    }
}
