﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using Pms.UserEvent.SystemEvent;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手摘表前参数准备，可能跳步
    /// </summary>
    public class Behavior_65072 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public WorkOrderService workOrderService { get; set; }

        public CacheService cacheService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65072() : base(65072)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            workOrderService = new WorkOrderService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            cacheService = new CacheService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            List<TrayBarcodeModel> listTrayBarcodeModel = trayBarcodeService.GetAllTrayBarcodeRemain(e.TaskBook.Carrier);
            if (listTrayBarcodeModel == null || listTrayBarcodeModel.Count <= 0)
            {
                e.IsSuccessed = false;
                return;
            }
            if (string.IsNullOrEmpty(listTrayBarcodeModel[0].TrayBarcode) || listTrayBarcodeModel[0].TrayBarcode.Contains("ERRO"))
            {
                e.IsSuccessed = false;
                return;
            }
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out InternalInterface.Enums.EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
            if (CSystemEvent.SingleWatchHoldingJaw == 3)
            {
                //20220620 add
                int allowMaxExcerptCount = 3;
                if ((carModel.SecondRollLayer == 0 && carModel.FirstTopLayerNumber == 9) || (carModel.FirstRollLayer == 6 && carModel.FirstTopLayerNumber == 9))//第2滚筒无空箱或第1滚筒6层且第1滚筒上层只剩最后一行空位置
                {
                    cacheService.SearchRemainStore(e.TaskBook.Carrier, orderData.Reserved2, 2, out int RowIndexF, out string[] BarcodeF);
                    if (BarcodeF != null)
                    {
                        allowMaxExcerptCount = allowMaxExcerptCount - BarcodeF.Length;
                    }
                }
                if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished })
                    && deceiveService.CanSingleExcerptWatchNG(e.TaskBook.WareHouse, orderData.Reserved2, out string[] Barcode, out int RowIndex, out int ColumnIndex, out int ExcerptCount, allowMaxExcerptCount)
                    && !cacheService.SearchFullStore(e.TaskBook.Carrier, orderData.Reserved2, 2, out int RowIndexB, out string[] BarcodeB) && carModel.FirstTopLayerNumber != 12)
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65072，逻辑值：{RowIndex}-{ColumnIndex}-0");

                    carModel.Barcode1 = Barcode[0]; carModel.Barcode2 = Barcode[1]; carModel.Barcode3 = Barcode[2]; carModel.PullNumber = ColumnIndex;
                    carModel.FirstRollMaterialType = deceiveService.GetMaterialType(e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 2, 2));
                    carModel.FirstRollBFinished = 1; carModel.FirstRollMaterialQuantity = 2; carModel.TaskNo = orderData.Reserved2;
                    deceiveService.UpdateSingleDeceiveBarcode(e.TaskBook.WareHouse, RowIndex == 4 ? 3 : RowIndex, new int[] { ColumnIndex & 0x01, ColumnIndex & 0x02, (ColumnIndex & 0x04) == 0x04 ? 3 : 0 }, new string[] { "0", "0", "0" }, 2, "0", 0);
                    carService.UpdateFullCarRoll(carModel); workOrderService.UpdateExcerptWatchNumber(orderData.Reserved2, ExcerptCount);//3
                }
                else
                {
                    e.IsDefaultNext = false;
                }
            }
            else
            {
                if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished })
                    && deceiveService.CanSingleExcerptWatchNG(e.TaskBook.WareHouse, orderData.Reserved2, out string[] Barcode, out int RowIndex, out int ColumnIndex, out int ExcerptCount, 1)
                    && !cacheService.SearchFullStore(e.TaskBook.Carrier, orderData.Reserved2, 2, out int RowIndexB, out string[] BarcodeB) && carModel.FirstTopLayerNumber != 12)
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65052，逻辑值：{RowIndex}-{ColumnIndex}-0");

                    carModel.Barcode1 = ColumnIndex == 1 ? Barcode[0] : ColumnIndex == 2 ? Barcode[1] : Barcode[2]; carModel.PullNumber = 1;//ColumnIndex
                    carModel.FirstRollMaterialType = deceiveService.GetMaterialType(e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 2, 2));
                    carModel.FirstRollBFinished = 1; carModel.FirstRollMaterialQuantity = 1; carModel.TaskNo = orderData.Reserved2;
                    deceiveService.UpdateSingleDeceiveBarcode(e.TaskBook.WareHouse, RowIndex, ColumnIndex, "0", 2, "0", 0);
                    carService.UpdateFullCarRoll(carModel); workOrderService.UpdateExcerptWatchNumber(orderData.Reserved2, ExcerptCount);//3
                }
                else
                {
                    e.IsDefaultNext = false;
                }
            }
            e.IsSuccessed = true;
        }
    }
}
