﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手扫托盘条码前参数准备，可能跳步
    /// </summary>
    public class Behavior_65030 : BaseStepBookEvent
    {
        public CarService carService { get; set; }

        public TaskService taskService { get; set; }

        public Behavior_65030() : base(65030)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            PocketWatchEnum[] pocketWatchEnum = new PocketWatchEnum[] { PocketWatchEnum.TaskException, PocketWatchEnum.TaskPointFinished, PocketWatchEnum.TaskFinished, PocketWatchEnum.WorkOrderFinished };
            if (!taskService.JudgementPWTaskStatus(orderData.Reserved1, pocketWatchEnum))
            {
                CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
                if (carModel.FirstTopLayerNumber == 12)
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { carModel.FirstRollLayer, 0, 0 }, out int failcode2);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65030，逻辑值：{carModel.FirstRollLayer}-0-0");
                }
                else
                {
                    e.IsDefaultNext = false;
                }
            }
            else
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65030，逻辑值：0-0-0");
            }
            e.IsSuccessed = true;
        }
    }
}
