﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using Pms.UserEvent.SystemEvent;
using System;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手抓车上表前参数准备，可能跳步
    /// </summary>
    public class Behavior_65032 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public WorkOrderService workOrderService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65032() : base(65032)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            workOrderService = new WorkOrderService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }

            PocketWatchEnum[] pocketWatchEnum = new PocketWatchEnum[] { PocketWatchEnum.TaskException, PocketWatchEnum.TaskPointFinished, PocketWatchEnum.TaskFinished, PocketWatchEnum.WorkOrderFinished };

            if (taskService.JudgementPWTaskStatus(orderData.Reserved1, pocketWatchEnum))
            {
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }
            if (workOrderService.JudegementPocketWatchBFinshed(carModel.TaskNo))//JudegementWorkOrderBFinshed
            {
                taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.WorkOrderFinished));
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }
            if (deceiveService.CanSinglePocketWatch(e.TaskBook.WareHouse) == 0)
            {
                taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.TaskPointFinished));
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }
            if (carModel.FirstRollLayer == 0)
            {
                taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.TaskFinished));
                e.IsDefaultNext = false; e.IsSuccessed = true; return;//20210715 add
            }
            if (carModel.FirstTopLayerNumber == 0)//20220621 add
            {
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }

            int RowIndex = Convert.ToInt32(Math.Pow(2, 4 - Convert.ToInt32(carModel.FirstTopLayerNumber / 3))), ColumnIndex = 7;//20220620 remark 需要考虑以实际扫码情况进行抓表控制

            //string tempBarcode1 = ""; 
            //string tempBarcode2 = "";
            //string tempBarcode3 = "";
            if (CSystemEvent.SingleWatchHoldingJaw == 3)
            {
                TrayBarcodeModel trayBarcodeModel = trayBarcodeService.GetTrayBarcode(e.TaskBook.Carrier);
                #region 新处理方法 20220622 add 
                string tempBarcode1 = "";
                string tempBarcode2 = "";
                string tempBarcode3 = "";
                ColumnIndex = 0;
                int remainCountToPocket = workOrderService.GetRemainCountNeedToPocket(carModel.TaskNo);
                int pocketcount = 0;
                bool enough = false;
                carModel.Barcode1 = "0";
                carModel.Barcode2 = "0";
                carModel.Barcode3 = "0";
                if (trayBarcodeModel.Barcode1 != "0" || trayBarcodeModel.Barcode2 != "0" || trayBarcodeModel.Barcode3 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode1;
                    if (carModel.Barcode1 != "0" && carModel.Barcode1 != "ERROR")
                    {
                        pocketcount++;
                        if (pocketcount >= remainCountToPocket)
                        {
                            enough = true;
                        }
                        trayBarcodeModel.Barcode1 = "0";
                    }
                    if (!enough)
                    {
                        carModel.Barcode2 = trayBarcodeModel.Barcode2;
                        if (carModel.Barcode2 != "0" && carModel.Barcode2 != "ERROR")
                        {
                            pocketcount++;
                            if (pocketcount >= remainCountToPocket)
                            {
                                enough = true;
                            }
                            trayBarcodeModel.Barcode2 = "0";
                        }
                        if (!enough)
                        {
                            carModel.Barcode3 = trayBarcodeModel.Barcode3;
                            if (carModel.Barcode3 != "0" && carModel.Barcode3 != "ERROR")
                            {
                                pocketcount++;
                            }
                            trayBarcodeModel.Barcode3 = "0";
                        }
                    }



                    RowIndex = 1; //ColumnIndex = 1;
                    tempBarcode1 = carModel.Barcode1;
                    tempBarcode2 = carModel.Barcode2;
                    tempBarcode3 = carModel.Barcode3;
                }
                else if (trayBarcodeModel.Barcode4 != "0" || trayBarcodeModel.Barcode5 != "0" || trayBarcodeModel.Barcode6 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode4;
                    if (carModel.Barcode1 != "0" && carModel.Barcode1 != "ERROR")
                    {
                        pocketcount++;
                        if (pocketcount >= remainCountToPocket)
                        {
                            enough = true;
                        }
                        trayBarcodeModel.Barcode4 = "0";
                    }
                    if (!enough)
                    {
                        carModel.Barcode2 = trayBarcodeModel.Barcode5;
                        if (carModel.Barcode2 != "0" && carModel.Barcode2 != "ERROR")
                        {
                            pocketcount++;
                            if (pocketcount >= remainCountToPocket)
                            {
                                enough = true;
                            }
                            trayBarcodeModel.Barcode5 = "0";
                        }
                        if (!enough)
                        {
                            carModel.Barcode3 = trayBarcodeModel.Barcode6;
                            if (carModel.Barcode3 != "0" && carModel.Barcode3 != "ERROR")
                            {
                                pocketcount++;
                            }
                            trayBarcodeModel.Barcode6 = "0";
                        }
                    }
                    RowIndex = 2; //ColumnIndex = 2;
                    tempBarcode1 = carModel.Barcode1;
                    tempBarcode2 = carModel.Barcode2;
                    tempBarcode3 = carModel.Barcode3;
                }
                else if (trayBarcodeModel.Barcode7 != "0" || trayBarcodeModel.Barcode8 != "0" || trayBarcodeModel.Barcode9 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode7;
                    if (carModel.Barcode1 != "0" && carModel.Barcode1 != "ERROR")
                    {
                        pocketcount++;
                        if (pocketcount >= remainCountToPocket)
                        {
                            enough = true;
                        }
                        trayBarcodeModel.Barcode7 = "0";
                    }
                    if (!enough)
                    {
                        carModel.Barcode2 = trayBarcodeModel.Barcode8;
                        if (carModel.Barcode2 != "0" && carModel.Barcode2 != "ERROR")
                        {
                            pocketcount++;
                            if (pocketcount >= remainCountToPocket)
                            {
                                enough = true;
                            }
                            trayBarcodeModel.Barcode8 = "0";
                        }
                        if (!enough)
                        {
                            carModel.Barcode3 = trayBarcodeModel.Barcode9;
                            if (carModel.Barcode3 != "0" && carModel.Barcode3 != "ERROR")
                            {
                                pocketcount++;
                            }
                            trayBarcodeModel.Barcode9 = "0";
                        }
                    }
                    RowIndex = 4; //ColumnIndex = 1;
                    tempBarcode1 = carModel.Barcode1;
                    tempBarcode2 = carModel.Barcode2;
                    tempBarcode3 = carModel.Barcode3;
                }
                else if (trayBarcodeModel.Barcode10 != "0" || trayBarcodeModel.Barcode1 != "0" || trayBarcodeModel.Barcode12 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode10;
                    if (carModel.Barcode1 != "0" && carModel.Barcode1 != "ERROR")
                    {
                        pocketcount++;
                        if (pocketcount >= remainCountToPocket)
                        {
                            enough = true;
                        }
                        trayBarcodeModel.Barcode10 = "0";
                    }
                    if (!enough)
                    {
                        carModel.Barcode2 = trayBarcodeModel.Barcode11;
                        if (carModel.Barcode2 != "0" && carModel.Barcode2 != "ERROR")
                        {
                            pocketcount++;
                            if (pocketcount >= remainCountToPocket)
                            {
                                enough = true;
                            }
                            trayBarcodeModel.Barcode11 = "0";
                        }
                        if (!enough)
                        {
                            carModel.Barcode3 = trayBarcodeModel.Barcode12;
                            if (carModel.Barcode3 != "0" && carModel.Barcode3 != "ERROR")
                            {
                                pocketcount++;
                            }
                            trayBarcodeModel.Barcode12 = "0";
                        }
                    }
                    RowIndex = 8; //ColumnIndex = 2;
                    tempBarcode1 = carModel.Barcode1;
                    tempBarcode2 = carModel.Barcode2;
                    tempBarcode3 = carModel.Barcode3;
                }


                if (carModel.Barcode1 != "0")
                {
                    ColumnIndex = ColumnIndex + 1;
                }
                if (carModel.Barcode2 != "0")
                {
                    ColumnIndex = ColumnIndex + 2;
                }
                if (carModel.Barcode3 != "0")
                {
                    ColumnIndex = ColumnIndex + 4;
                }
                #endregion
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, carModel.FirstRollLayer }, out int failcode1);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65032，逻辑值：{RowIndex}-{ColumnIndex}-{carModel.FirstRollLayer}");
                #region 旧处理方法
                //switch (RowIndex)
                //{
                //    case 1:
                //        carModel.Barcode1 = trayBarcodeModel.Barcode1;
                //        carModel.Barcode2 = trayBarcodeModel.Barcode2;
                //        carModel.Barcode3 = trayBarcodeModel.Barcode3;
                //        trayBarcodeModel.Barcode1 = "0"; trayBarcodeModel.Barcode2 = "0"; trayBarcodeModel.Barcode3 = "0";
                //        break;
                //    case 2:
                //        carModel.Barcode1 = trayBarcodeModel.Barcode4;
                //        carModel.Barcode2 = trayBarcodeModel.Barcode5;
                //        carModel.Barcode3 = trayBarcodeModel.Barcode6;
                //        trayBarcodeModel.Barcode4 = "0"; trayBarcodeModel.Barcode5 = "0"; trayBarcodeModel.Barcode6 = "0";
                //        break;
                //    case 4:
                //        carModel.Barcode1 = trayBarcodeModel.Barcode7;
                //        carModel.Barcode2 = trayBarcodeModel.Barcode8;
                //        carModel.Barcode3 = trayBarcodeModel.Barcode9;
                //        trayBarcodeModel.Barcode7 = "0"; trayBarcodeModel.Barcode8 = "0"; trayBarcodeModel.Barcode9 = "0";
                //        break;
                //    case 8:
                //        carModel.Barcode1 = trayBarcodeModel.Barcode10;
                //        carModel.Barcode2 = trayBarcodeModel.Barcode11;
                //        carModel.Barcode3 = trayBarcodeModel.Barcode12;
                //        trayBarcodeModel.Barcode10 = "0"; trayBarcodeModel.Barcode11 = "0"; trayBarcodeModel.Barcode12 = "0";
                //        break;
                //}
                #endregion

                Model.TrayBarcodeModel trayBarcodeModelCurrent = trayBarcodeService.GetTrayBarcode(carModel.CarID);
                TrayBarcodeModel trayCodeModelRemain = trayBarcodeService.GetTrayBarcodeRemain(100, trayBarcodeModelCurrent.TrayBarcode);//GetTrayBarcodeRemain
                if (trayCodeModelRemain != null)
                {
                    if (trayCodeModelRemain.Barcode1 == carModel.Barcode1 || trayCodeModelRemain.Barcode1 == carModel.Barcode2 || trayCodeModelRemain.Barcode1 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode1 = "0";
                    }
                    if (trayCodeModelRemain.Barcode2 == carModel.Barcode1 || trayCodeModelRemain.Barcode2 == carModel.Barcode2 || trayCodeModelRemain.Barcode2 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode2 = "0";
                    }
                    if (trayCodeModelRemain.Barcode3 == carModel.Barcode1 || trayCodeModelRemain.Barcode3 == carModel.Barcode2 || trayCodeModelRemain.Barcode3 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode3 = "0";
                    }
                    if (trayCodeModelRemain.Barcode4 == carModel.Barcode1 || trayCodeModelRemain.Barcode4 == carModel.Barcode2 || trayCodeModelRemain.Barcode4 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode4 = "0";
                    }
                    if (trayCodeModelRemain.Barcode5 == carModel.Barcode1 || trayCodeModelRemain.Barcode5 == carModel.Barcode2 || trayCodeModelRemain.Barcode5 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode5 = "0";
                    }
                    if (trayCodeModelRemain.Barcode6 == carModel.Barcode1 || trayCodeModelRemain.Barcode6 == carModel.Barcode2 || trayCodeModelRemain.Barcode6 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode6 = "0";
                    }
                    if (trayCodeModelRemain.Barcode7 == carModel.Barcode1 || trayCodeModelRemain.Barcode7 == carModel.Barcode2 || trayCodeModelRemain.Barcode7 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode7 = "0";
                    }
                    if (trayCodeModelRemain.Barcode8 == carModel.Barcode1 || trayCodeModelRemain.Barcode8 == carModel.Barcode2 || trayCodeModelRemain.Barcode8 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode8 = "0";
                    }
                    if (trayCodeModelRemain.Barcode9 == carModel.Barcode1 || trayCodeModelRemain.Barcode9 == carModel.Barcode2 || trayCodeModelRemain.Barcode9 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode9 = "0";
                    }
                    if (trayCodeModelRemain.Barcode10 == carModel.Barcode1 || trayCodeModelRemain.Barcode10 == carModel.Barcode2 || trayCodeModelRemain.Barcode10 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode10 = "0";
                    }
                    if (trayCodeModelRemain.Barcode11 == carModel.Barcode1 || trayCodeModelRemain.Barcode11 == carModel.Barcode2 || trayCodeModelRemain.Barcode11 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode11 = "0";
                    }
                    if (trayCodeModelRemain.Barcode12 == carModel.Barcode1 || trayCodeModelRemain.Barcode12 == carModel.Barcode2 || trayCodeModelRemain.Barcode12 == carModel.Barcode3)
                    {
                        trayCodeModelRemain.Barcode12 = "0";
                    }
                    trayBarcodeService.UpdateTrayBarcodeRemain(trayCodeModelRemain);
                }

                carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber - (ColumnIndex == 7 ? 3 : (ColumnIndex == 3 || ColumnIndex == 5 || ColumnIndex == 6) ? 2 : (ColumnIndex == 1 || ColumnIndex == 2 || ColumnIndex == 4) ? 1 : 0); carModel.PocketNumber = ColumnIndex;
                carService.UpdateFullCarRoll(carModel); trayBarcodeService.UpdateTrayBarcode(trayBarcodeModel);
            }
            else
            {
                TrayBarcodeModel trayBarcodeModel = trayBarcodeService.GetTrayBarcode(e.TaskBook.Carrier);//根据实际情况进行抓表控制
                string tempBarcode = "";
                #region 新处理方法
                if (trayBarcodeModel.Barcode1 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode1;
                    trayBarcodeModel.Barcode1 = "0";
                    RowIndex = 1; ColumnIndex = 1;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode2 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode2;
                    trayBarcodeModel.Barcode2 = "0";
                    RowIndex = 1; ColumnIndex = 2;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode3 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode3;//4
                    trayBarcodeModel.Barcode3 = "0";
                    RowIndex = 1; ColumnIndex = 4;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode4 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode4;//5
                    trayBarcodeModel.Barcode4 = "0";
                    RowIndex = 2; ColumnIndex = 1;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode5 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode5;
                    trayBarcodeModel.Barcode5 = "0";
                    RowIndex = 2; ColumnIndex = 2;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode6 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode6;//4
                    trayBarcodeModel.Barcode6 = "0";
                    RowIndex = 2; ColumnIndex = 4;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode7 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode7;//5
                    trayBarcodeModel.Barcode7 = "0";
                    RowIndex = 4; ColumnIndex = 1;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode8 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode8;
                    trayBarcodeModel.Barcode8 = "0";
                    RowIndex = 4; ColumnIndex = 2;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode9 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode9;//4
                    trayBarcodeModel.Barcode9 = "0";
                    RowIndex = 4; ColumnIndex = 4;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode10 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode10;//5
                    trayBarcodeModel.Barcode10 = "0";
                    RowIndex = 8; ColumnIndex = 1;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode11 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode11;//4
                    trayBarcodeModel.Barcode11 = "0";
                    RowIndex = 8; ColumnIndex = 2;
                    tempBarcode = carModel.Barcode1;
                }
                else if (trayBarcodeModel.Barcode12 != "0")
                {
                    carModel.Barcode1 = trayBarcodeModel.Barcode12;//5
                    trayBarcodeModel.Barcode12 = "0";
                    RowIndex = 8; ColumnIndex = 4;
                    tempBarcode = carModel.Barcode1;
                }
                #endregion
                TrayBarcodeModel trayBarcodeModelRemain = trayBarcodeService.GetTrayBarcodeByBarcodeRemain(100, tempBarcode);//GetTrayBarcodeRemain
                if (trayBarcodeModelRemain != null)
                {
                    if (trayBarcodeModelRemain.Barcode1 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode1 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode2 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode2 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode3 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode3 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode4 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode4 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode5 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode5 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode6 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode6 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode7 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode7 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode8 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode8 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode9 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode9 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode10 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode10 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode11 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode11 = "0";
                    }
                    else if (trayBarcodeModelRemain.Barcode12 == tempBarcode)
                    {
                        trayBarcodeModelRemain.Barcode12 = "0";
                    }
                    trayBarcodeService.UpdateTrayBarcodeRemain(trayBarcodeModelRemain);
                }
                if (ColumnIndex != 7 && ColumnIndex != 0)
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, carModel.FirstRollLayer }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65032，逻辑值：{RowIndex}-{ColumnIndex}-{carModel.FirstRollLayer}");
                    carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber - 1; carModel.PocketNumber = 1;
                    carService.UpdateFullCarRoll(carModel); trayBarcodeService.UpdateTrayBarcode(trayBarcodeModel);
                }
                else
                {
                    e.IsSuccessed = false; return;
                }
            }

            e.IsSuccessed = true;
        }
    }
}
