﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手抓空托盘前参数准备，可能跳步
    /// </summary>
    public class Behavior_65035 : BaseStepBookEvent
    {
        public CarService carService { get; set; }

        public TaskService taskService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65035() : base(65035)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
            {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
            if (carModel.FirstTopLayerNumber == 0)
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { carModel.FirstRollLayer, 1, carModel.SecondRollLayer + 1}, out int failcode1);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65035，逻辑值：{carModel.FirstRollLayer}-1-{carModel.SecondRollLayer + 1}");

                carModel.FirstRollLayer = carModel.FirstRollLayer - 1;
                if (carModel.FirstRollLayer != 0) { carModel.FirstTopLayerNumber = 12; }
                carModel.SecondRollLayer = carModel.SecondRollLayer + 1;
                carModel.SecondRollMaterialType = carModel.FirstRollMaterialType;
                if (carModel.FirstRollLayer == 0) { carModel.FirstRollMaterialType = 0; }
                carModel.SecondRollBFinished = 0; carModel.SecondRollMaterialQuantity = 0;
                string[] PileNoArr = carModel.FirstPileNo.Split('_'); carModel.FirstPileNo = "";
                TrayBarcodeModel trayBarcodeModel = trayBarcodeService.GetTrayBarcode(e.TaskBook.Carrier);
                carModel.SecondPileNo = carModel.SecondPileNo == "0" ? trayBarcodeModel.TrayBarcode : carModel.SecondPileNo + "_" + trayBarcodeModel.TrayBarcode;
                for (int i = 0; i < PileNoArr.Length; i++)
                {
                    if (PileNoArr[i] != trayBarcodeModel.TrayBarcode)
                    {
                        if (carModel.FirstPileNo.Length == 0)
                        {
                            carModel.FirstPileNo = PileNoArr[i];
                        }
                        else
                        {
                            carModel.FirstPileNo = carModel.FirstPileNo + "_" + PileNoArr[i]; 
                        }
                    }
                }
                carService.UpdateFullCarRoll(carModel);
            }
            else
            {
                e.IsDefaultNext = false;
            }

            e.IsSuccessed = true;
        }
    }
}
