﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手扫托盘条码前参数准备
    /// </summary>
    public class Behavior_65081 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65081() : base(65081)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
            if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished }))
            {
                if (deceiveService.CanThreeExcerptWatchOK(e.TaskBook.WareHouse, orderData.Reserved2, out string Barcode, out int RowIndex, out int ColumnIndex))
                {
                    if (carModel.FirstTopLayerNumber == 0)
                    {
                        CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { carModel.FirstRollLayer, 0, 0 }, out int failcode1);
                        CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65081，逻辑值：{carModel.FirstRollLayer}-0-0");
                    }
                    else
                    {
                        CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                        CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65081，逻辑值：0-0-0");
                    }
                }
                else
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65081，逻辑值：0-0-0");
                    taskService.UpdateEWTaskStatus(orderData.Reserved1, (int)ExcerptWatchEnum.TaskFinished);
                }
            }
            else
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65081，逻辑值：0-0-0");
            }
        }
    }
}
