﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using System;
using System.Collections.Generic;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手摘表前参数准备，可能跳步
    /// </summary>
    public class Behavior_65082 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public WorkOrderService workOrderService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65082() : base(65082)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            workOrderService = new WorkOrderService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            List<TrayBarcodeModel> listTrayBarcodeModel = trayBarcodeService.GetAllTrayBarcodeRemain(e.TaskBook.Carrier);
            if(listTrayBarcodeModel==null||listTrayBarcodeModel.Count<=0)
            {
                e.IsSuccessed = false;
                return;
            }
            if(string.IsNullOrEmpty(listTrayBarcodeModel[0].TrayBarcode)|| listTrayBarcodeModel[0].TrayBarcode.Contains("ERRO"))
            {
                e.IsSuccessed = false;
                return;
            }
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
            if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished }))
            {
                if (deceiveService.CanThreeExcerptWatchOK(e.TaskBook.WareHouse, orderData.Reserved2, out string Barcode, out int RowIndex, out int ColumnIndex) && carModel.SecondTopLayerNumber != 4)
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65082，逻辑值：{RowIndex}-{ColumnIndex}-0");

                    carModel.FirstRollMaterialType = deceiveService.GetMaterialType(e.TaskBook.WareHouse.Remove(e.TaskBook.WareHouse.Length - 2, 2));
                    carModel.Barcode1 = Barcode; carModel.PullNumber = ColumnIndex; carModel.FirstRollBFinished = 1; carModel.FirstRollMaterialQuantity = 1; carModel.TaskNo = orderData.Reserved2;
                    deceiveService.UpdateThreeDeceiveBarcode(e.TaskBook.WareHouse, RowIndex, ColumnIndex, "0", 2, "0", 0);
                    carService.UpdateFullCarRoll(carModel); workOrderService.UpdateExcerptWatchNumber(orderData.Reserved2, 1);//carModel.PullNumber
                }
                else
                {
                    e.IsDefaultNext = false;
                }
            }
            else
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65082，逻辑值：0-0-0");
            }
            e.IsSuccessed = true;
        }
    }
}
