﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using System;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手放表到托盘前参数准备
    /// </summary>
    public class Behavior_65083 : BaseStepBookEvent
    {
        public CarService carService { get; set; }

        public TaskService taskService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65083() : base(65083)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
            if (!taskService.JudgementEWTaskStatus(e.TaskBook.Reserved1, new ExcerptWatchEnum[] { ExcerptWatchEnum.TaskPointFinished, ExcerptWatchEnum.TaskFinished }))
            {
                if (carModel.FirstTopLayerNumber != 4 && carModel.PullNumber != 0)
                {
                    int RowIndex = Convert.ToInt32(carModel.FirstTopLayerNumber / 2) + 1, ColumnIndex = Convert.ToInt32(carModel.FirstTopLayerNumber % 2) + 1;
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, carModel.FirstRollLayer }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65083，逻辑值：{RowIndex}-{ColumnIndex}-0");

                    TrayBarcodeModel trayBarcodeModel = trayBarcodeService.GetTrayBarcode(e.TaskBook.Carrier);
                    switch (carModel.FirstTopLayerNumber)
                    {
                        case 0: trayBarcodeModel.Barcode1 = carModel.Barcode1; break;
                        case 1: trayBarcodeModel.Barcode2 = carModel.Barcode1; break;
                        case 2: trayBarcodeModel.Barcode3 = carModel.Barcode1; break;
                        case 3: trayBarcodeModel.Barcode4 = carModel.Barcode1; break;
                    }
                    carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber + 1; carModel.PullNumber = 0; carModel.Barcode1 = "0";
                    carService.UpdateFullCarRoll(carModel); trayBarcodeService.UpdateTrayBarcode(trayBarcodeModel);
                }
                else
                {
                    CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                    CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65083，逻辑值：0-0-0");
                }
            }
            else
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode1);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65083，逻辑值：0-0-0");
            }
            e.IsSuccessed = true;
        }
    }
}
