﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;
using System;
using System.Threading;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手抓车上表前参数准备，可能跳步
    /// </summary>
    public class Behavior_65042 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public CarService carService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public WorkOrderService workOrderService { get; set; }

        public TrayBarcodeService trayBarcodeService { get; set; }

        public Behavior_65042() : base(65042)
        {

        }
        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            trayBarcodeService = new TrayBarcodeService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            workOrderService = new WorkOrderService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);
            CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }

            PocketWatchEnum[] pocketWatchEnum = new PocketWatchEnum[] { PocketWatchEnum.TaskException, PocketWatchEnum.TaskPointFinished, PocketWatchEnum.TaskFinished, PocketWatchEnum.WorkOrderFinished };

            if (taskService.JudgementPWTaskStatus(orderData.Reserved1, pocketWatchEnum))
            {
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }
            if (workOrderService.JudegementPocketWatchBFinshed(carModel.TaskNo))//JudegementWorkOrderBFinshed
            {
                taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.WorkOrderFinished));
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }
            if (!deceiveService.CanThreePocketWatch(e.TaskBook.WareHouse, out int RowIndex, out int ColumnIndex))
            {
                taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.TaskPointFinished));
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }
            if (carModel.FirstRollLayer == 0)
            {
                taskService.UpdatePWTaskStatus(orderData.Reserved1, Convert.ToInt32(PocketWatchEnum.TaskFinished));
                e.IsDefaultNext = false; e.IsSuccessed = true; return;
            }

            TrayBarcodeModel trayBarcodeModel = trayBarcodeService.GetTrayBarcode(e.TaskBook.Carrier);//根据实际情况进行抓表控制
            string tempBarcode = "";
            #region 新处理方法
            if (trayBarcodeModel.Barcode1!="0")
            {
                carModel.Barcode1 = trayBarcodeModel.Barcode1;
                trayBarcodeModel.Barcode1 = "0";
                RowIndex = 1; ColumnIndex = 1;
                tempBarcode = carModel.Barcode1;
            }
            else if (trayBarcodeModel.Barcode2 != "0")
            {
                carModel.Barcode1 = trayBarcodeModel.Barcode2;
                trayBarcodeModel.Barcode2 = "0";
                RowIndex = 1; ColumnIndex = 2;
                tempBarcode = carModel.Barcode1;
            }
            else if (trayBarcodeModel.Barcode3 != "0")
            {
                carModel.Barcode1 = trayBarcodeModel.Barcode3;//4
                trayBarcodeModel.Barcode3 = "0";
                RowIndex = 2; ColumnIndex = 1;
                tempBarcode = carModel.Barcode1;
            }
            else if (trayBarcodeModel.Barcode4 != "0")
            {
                carModel.Barcode1 = trayBarcodeModel.Barcode4;//5
                trayBarcodeModel.Barcode4 = "0";
                RowIndex = 2; ColumnIndex = 2;
                tempBarcode = carModel.Barcode1;
            }
            #endregion
            #region 旧处理方法
            //switch (carModel.FirstTopLayerNumber)
            //{
            //    case 4:
            //        carModel.Barcode1 = trayBarcodeModel.Barcode1;
            //        trayBarcodeModel.Barcode1 = "0";
            //        RowIndex = 1; ColumnIndex = 1;
            //        tempBarcode = carModel.Barcode1;
            //        break;
            //    case 3:
            //        carModel.Barcode1 = trayBarcodeModel.Barcode2;
            //        trayBarcodeModel.Barcode2 = "0";
            //        RowIndex = 1; ColumnIndex = 2;
            //        tempBarcode = carModel.Barcode1;
            //        break;
            //    case 2:
            //        carModel.Barcode1 = trayBarcodeModel.Barcode4;//4
            //        trayBarcodeModel.Barcode4 = "0";
            //        RowIndex = 2; ColumnIndex = 1;
            //        tempBarcode = carModel.Barcode1;
            //        break;
            //    case 1:
            //        carModel.Barcode1 = trayBarcodeModel.Barcode5;//5
            //        trayBarcodeModel.Barcode5 = "0";
            //        RowIndex = 2; ColumnIndex = 2;
            //        tempBarcode = carModel.Barcode1;
            //        break;
            //}
            #endregion

            //65032也要加
            TrayBarcodeModel trayBarcodeModelRemain = trayBarcodeService.GetTrayBarcodeByBarcodeRemain(100, tempBarcode);//GetTrayBarcodeRemain
            if (trayBarcodeModelRemain != null)
            {
                if (trayBarcodeModelRemain.Barcode1 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode1 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode2 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode2 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode3 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode3 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode4 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode4 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode5 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode5 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode6 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode6 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode7 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode7 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode8 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode8 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode9 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode9 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode10 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode10 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode11 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode11 = "0";
                }
                else if (trayBarcodeModelRemain.Barcode12 == tempBarcode)
                {
                    trayBarcodeModelRemain.Barcode12 = "0";
                }
                trayBarcodeService.UpdateTrayBarcodeRemain(trayBarcodeModelRemain);
            }
            CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, carModel.FirstRollLayer }, out int failcode1);
            CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65042，逻辑值：{RowIndex}-{ColumnIndex}-{carModel.FirstRollLayer}");
            carModel.FirstTopLayerNumber = carModel.FirstTopLayerNumber - 1; carModel.PocketNumber = 1;
            carService.UpdateFullCarRoll(carModel); trayBarcodeService.UpdateTrayBarcode(trayBarcodeModel);
            e.IsSuccessed = true;
        }
    }
}
