﻿using CasunCommon.SystemLog;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using Pms.InternalInterface.Enums;
using Pms.InternalInterface.Models;
using Pms.UserEvent.Config;
using Pms.UserEvent.EnumType;
using Pms.UserEvent.Model;
using Pms.UserEvent.Service;

namespace Pms.UserEvent.StepBookEvent
{
    /// <summary>
    /// 机械手挂表前参数准备
    /// </summary>
    public class Behavior_65043 : BaseStepBookEvent
    {
        public TaskService taskService { get; set; }

        public DeceiveService deceiveService { get; set; }

        public CarService carService { get; set; }

        public WorkOrderService workOrderService { get; set; }

        public Behavior_65043() : base(65043)
        {

        }

        protected override void StepBookStarted(object sender, P_StepBookExecuteEventArgs e)
        {
            taskService = new TaskService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            deceiveService = new DeceiveService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            carService = new CarService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
            workOrderService = new WorkOrderService(CConfigManager.gs_globalConfig.PMSDataBaseUrl);
        }

        protected override void StepBookEnd(object sender, P_StepBookExecuteEventArgs e)
        {
            OrderData orderData = CPmsControllers.Instance.OrderDataController.GetOrder(e.TaskBook.OrderID, out string strMsg, out EFailCode failCode);

            PocketWatchEnum[] pocketWatchEnum = new PocketWatchEnum[] { PocketWatchEnum.TaskException, PocketWatchEnum.TaskPointFinished, PocketWatchEnum.TaskFinished, PocketWatchEnum.WorkOrderFinished };
            if (!taskService.JudgementPWTaskStatus(orderData.Reserved1, pocketWatchEnum))
            {
                CarModel carModel = carService.GetCar(e.TaskBook.Carrier); if (carModel == null) { e.IsSuccessed = false; return; }
                deceiveService.CanThreePocketWatch(e.TaskBook.WareHouse, out int RowIndex, out int ColumnIndex);
                if (RowIndex == 0 || ColumnIndex == 0 || carModel.PocketNumber == 0) { e.IsSuccessed = false; return; }
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { RowIndex, ColumnIndex, 0 }, out int failcode);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65043，逻辑值：{RowIndex}-{ColumnIndex}-0");

                deceiveService.UpdateThreeDeceiveBarcode(e.TaskBook.WareHouse, RowIndex, ColumnIndex, carModel.Barcode1, 1, carModel.TaskNo, 0);
                carModel.PocketNumber = 0; carModel.Barcode1 = "0"; carService.UpdateFullCarRoll(carModel);
                workOrderService.UpdatePocketWatchNumber(carModel.TaskNo, 1);
            }
            else
            {
                CPmsControllers.Instance.CarrierController.WriteLogicValues(e.TaskBook.Carrier, 20, new int[] { 0, 0, 0 }, out int failcode);
                CLog.Instance.WriteLog("VcuLogicValue", $"任务编号：{e.TaskBook.OrderID}，小车编号：{e.TaskBook.Carrier}，操作码：65043，逻辑值：0-0-0");
            }
            e.IsSuccessed = true;
        }
    }
}
