﻿using MaxCommunication.DataBase.SQLServer;
using Pms.Interface;
using Pms.Interface.ResultModels;
using Pms.WebApiHost.Models;
using System;
using System.Collections.Generic;
using System.Data;
using System.Threading;
using System.Web.Http;
using System.Web.Http.Results;

namespace Pms.WebApiHost.Controllers
{
    //[RoutePrefix("AgvWebApi")]
    public class AgvController : ApiController
    {
        CPmsInterface _PmsInterface = new CPmsInterface();

        /// <summary>
        /// 任务添加接口
        /// </summary>
        /// <param name="sRequestTaskInfo"></param>
        //[Route("OrderAdd")]
        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> OrderAdd([FromBody]RequestOrderInfo sRequestOrderInfo)
        {
            Result res = new Result();

            DateTime CallTime;
            if (!DateTime.TryParse(sRequestOrderInfo.CallTime, out CallTime))
            {
                res.resCode = (int)EResultCode.ParameterError;
                res.resMsg = "ParameterError：Calledtime format error！";
                return Json(res);
            }
            string[] PosArr;
            PosArr = sRequestOrderInfo.PosArr.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

            OrderInfo sOrderInfo = new OrderInfo();
            sOrderInfo.OrderID = sRequestOrderInfo.OrderID;
            sOrderInfo.ParentOrderID = sRequestOrderInfo.ParentOrderID;
            sOrderInfo.OrderPriority = sRequestOrderInfo.Priority;
            sOrderInfo.CarrierType = sRequestOrderInfo.CarrierType;

            sOrderInfo.OrderType = sRequestOrderInfo.Type;

            sOrderInfo.CalledTime = CallTime;
            sOrderInfo.PosArr = PosArr;
            sOrderInfo.PosArrLength = sRequestOrderInfo.PosArrLength;
            sOrderInfo.OtherOrderData = sRequestOrderInfo.OtherOrderData;

            return Json(_PmsInterface.OrderAdd(sOrderInfo));
        }

        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> MidOrderAdd([FromBody]RequestMidOrder sRequsetOrder)
        {
            Result res = new Result();

            res.ResTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            res.BizCode = EBizCode.MidOrderAdd.ToString();
            res.resData = "";

            if (sRequsetOrder.OrderID == "" || sRequsetOrder.OrderID == null)
            {
                res.resCode = (int)EResultCode.ParameterError;
                //res.resMsg = "参数错误：OrderID不能为空!";
                res.resMsg = "ParameterError：OrderID can't be empty!";
                return Json(res);
            }

            DateTime CallTime;
            if (!DateTime.TryParse(sRequsetOrder.CallTime, out CallTime))
            {
                res.resCode = (int)EResultCode.ParameterError;
                //res.resMsg = "参数错误：CalledTime时间格式错误！";
                res.resMsg = "ParameterError：Calledtime format error！";
                return Json(res);
            }

            MidOrderInfo sMidOrder = new MidOrderInfo();

            sMidOrder.State = "Default"; // DEFAULT 状态 数据还未经过初处理。
            sMidOrder.OrderID = sRequsetOrder.OrderID;
            sMidOrder.Priority = sRequsetOrder.Priority;

            sMidOrder.Type = sRequsetOrder.Type;
            sMidOrder.CalledTime = CallTime;
            sMidOrder.CreateTime = DateTime.Now;
            sMidOrder.PosQue = sRequsetOrder.PosArr;
            sMidOrder.PosQueLength = sRequsetOrder.PosArrLength;
            sMidOrder.CarrierType = sRequsetOrder.CarrierType; // 任务添加的小车类型 任务处理的时候修改

            res = _PmsInterface.MidOrderAdd(sMidOrder);

            return Json(res);
        }

        /// <summary>
        /// 任务取消接口
        /// </summary>
        /// <param name="OrderID"></param>
        /// <returns>false: 取消成功 true：取消请求接收成功</returns>        
        //[Route("OrderCancel")]
        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> OrderCancel([FromBody]string OrderID)
        {
            return Json(_PmsInterface.OrderCancel(OrderID));
        }

        /// <summary>
        /// 获取订单状态
        /// </summary>
        /// <param name="OrderID"></param>
        /// <returns></returns>
        //[Route("GetOrder")]
        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> GetOrder([FromBody]string OrderID)
        {
            return Json(_PmsInterface.GetOrder(OrderID));
        }

        /// <summary>
        /// 获取当前所有执行中任务（历史任务除外）
        /// </summary>
        /// <returns></returns>
        //[Route("GetCurrentOrder")]
        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> GetCurrentOrder()
        {
            return Json(_PmsInterface.GetCurrentOrder());
        }

        /// <summary>
        /// 通过AgvID 获取AGV状态
        /// </summary>
        /// <returns></returns>
        //[Route("GetCarrier")]
        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> GetCarrier([FromBody]int CarrierID)
        {
            return Json(_PmsInterface.GetCarrier(CarrierID));
        }

        /// <summary>
        /// 获取所有AGV状态
        /// </summary>
        /// <returns></returns>
        [AcceptVerbs("Get", "POST")]
        public JsonResult<Result> GetAllCarrier()
        {
            return Json(_PmsInterface.GetAllCarrier());
        }

        [AcceptVerbs("Get", "POST")]
        public JsonResult<ResultModel> SetVerificationTaskinfo([FromBody] WorkOrderModel model)
        {
            ResultModel resultModel = new ResultModel();
            SQLServerExtension sqlServerExtension = new SQLServerExtension("server=.;uid=sa;pwd=123;database=PMS_KLDZ_6324");
            DataTable dt = sqlServerExtension.QueryTable("PMS_Max_WorkOrder", $"where WorkOrderID='{model.TASK_NO}'");
            if (dt != null && dt.Rows.Count != 0)
            {
                if (sqlServerExtension.UpdateRecord("PMS_Max_WorkOrder", $"TaskStatus = {model.STATUS},DateTime='{model.WRITE_DATE}'", $"where WorkOrderID = '{model.TASK_NO}'"))
                {
                    resultModel.RESULT_FLAG = "1";
                }
            }
            else
            {
                string values = $"'{model.TASK_NO}',{model.STATUS},{model.EQUIP_QTY},0,0,'{model.WRITE_DATE}'";
                if (sqlServerExtension.AddRecord("PMS_Max_WorkOrder", values))
                {
                    resultModel.RESULT_FLAG = "1";
                }
            }
            sqlServerExtension.Disconnect();
            return Json(resultModel);
        }

        [AcceptVerbs("Get", "POST")]
        public JsonResult<ResultModel> getWHArrival([FromBody] PocketWatchModel model)
        {
           ResultModel resultModel = new ResultModel();
            SQLServerExtension sqlServerExtension = new SQLServerExtension("server=.;uid=sa;pwd=123;database=PMS_KLDZ_6324");
            DataTable dataTable = sqlServerExtension.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveName = '{model.PLATFORM_NO}'");
            if (dataTable != null && dataTable.Rows.Count != 0)
            {
                string WorkStationName = dataTable.Rows[0]["WorkStationName"]?.ToString();
                string[] arryPileNo = null;
                if (model.PILE_NO.Contains("_"))
                {
                    arryPileNo = model.PILE_NO.Split('_');
                }
                else
                {
                    arryPileNo = new string[] { model.PILE_NO };
                }
                int tempLayerNumber = string.IsNullOrEmpty(model.PILE_NO) || model.PILE_NO == "0" ? 6 : arryPileNo.Length;
                int.TryParse(model.METER_TYPE, out int tempMETER_TYPE);
                bool emptyBoxOut = false;
                #region 判断是否为空箱
                if (model.BOX_METER == null || model.BOX_METER.Length < 1|| model.BOX_METER[0].METER==null|| model.BOX_METER[0].METER.Length<1)
                {
                    //空箱搬运到缓存位
                    emptyBoxOut = true;
                    string TrayBarcodeArr = "";
                    string[] TrayBarcodes= model.PILE_NO.Split('_');
                    foreach (var item in TrayBarcodes)
                    {
                        if (string.IsNullOrEmpty(TrayBarcodeArr))
                        {
                            TrayBarcodeArr = $"'{item}'";
                        }
                        else
                        {
                            TrayBarcodeArr = TrayBarcodeArr + $",'{item}'";
                        }
                    }
                    sqlServerExtension.DeleteRecord("PMS_Max_TrayBarcode",$"where and CarID=100 and TrayBarcode in ({TrayBarcodeArr})");
                    sqlServerExtension.Disconnect();
                }
                else
                {
                    foreach (var item in model.BOX_METER)
                    {
                        if (item.METER != null && item.METER.Length > 0)
                        {

                            string[] Barcodes = new string[] { "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0" };
                            //if (model.METER_TYPE == "01")
                            //{
                            foreach (var barcode in item.METER)
                            {
                                if (int.TryParse(barcode.METER_ID, out int tempMETER_ID) && tempMETER_ID > 0)
                                {
                                    Barcodes[tempMETER_ID - 1] = barcode.METER_CODE;
                                }
                            }
                            //}
                            //else
                            //{
                            //    foreach (var barcode in item.METER)
                            //    {
                            //        if (int.TryParse(barcode.METER_ID, out int tempMETER_ID) && tempMETER_ID > 0)
                            //        {
                            //            Barcodes[tempMETER_ID - 1] = barcode.METER_CODE;
                            //        }
                            //    }
                            //}
                            DataTable dataTableTrayBarcode = sqlServerExtension.QueryTable("PMS_Max_TrayBarcode", $"where CarID = {100} and TrayBarcode= '{item.BOX_NO}'");
                            if (dataTableTrayBarcode == null || dataTableTrayBarcode.Rows.Count == 0)
                            {
                                string valuesTrayBarcode = $"{100},'{item.BOX_NO}','{Barcodes[0]}','{Barcodes[1]}','{Barcodes[2]}','{Barcodes[3]}','{Barcodes[4]}','{Barcodes[5]}','{Barcodes[6]}','{Barcodes[7]}','{Barcodes[8]}','{Barcodes[9]}','{Barcodes[10]}','{Barcodes[11]}'";
                                sqlServerExtension.AddRecord("PMS_Max_TrayBarcode", valuesTrayBarcode); sqlServerExtension.Disconnect();
                            }
                            else
                            {
                                sqlServerExtension.UpdateRecord("PMS_Max_TrayBarcode",
                    $"Barcode1 = '{Barcodes[0]}',Barcode2 = '{Barcodes[1]}',Barcode3 = '{Barcodes[2]}'," +
                    $"Barcode4 = '{Barcodes[3]}',Barcode5 = '{Barcodes[4]}',Barcode6 = '{Barcodes[5]}'," +
                    $"Barcode7 = '{Barcodes[6]}',Barcode8 = '{Barcodes[7]}',Barcode9 = '{Barcodes[8]}'," +
                    $"Barcode10 = '{Barcodes[9]}',Barcode11 = '{Barcodes[10]}',Barcode12 = '{Barcodes[11]}'",
                    $"where TrayBarcode = '{item.BOX_NO}' and CarID=100");

                                sqlServerExtension.Disconnect();
                            }
                        }
                    }
                }
                #endregion
                int tempTopLayerNumber = 0;
                if (model.METER_TYPE == "01")
                {
                    if(!emptyBoxOut)
                    {
                        tempTopLayerNumber = 12;
                    }
                    string content = $"TaskNo = '{model.TASK_NO}',PileNo = '{model.PILE_NO}',MaterialType = {tempMETER_TYPE},LayerNumber = {tempLayerNumber},TopLayerNumber = {tempTopLayerNumber},Quantity = 0,BFinished = 0";
                    sqlServerExtension.UpdateRecord("PMS_Max_Export", content, $"where ExportName = '{WorkStationName}'");
                }
                else if (model.METER_TYPE == "02" || model.METER_TYPE == "03" || model.METER_TYPE == "04" || model.METER_TYPE == "05" || model.METER_TYPE == "06")
                {
                    if (!emptyBoxOut)
                    {
                        tempTopLayerNumber = 4;
                    }
                    string content = $"TaskNo = '{model.TASK_NO}',PileNo = '{model.PILE_NO}',MaterialType = {tempMETER_TYPE},LayerNumber = {tempLayerNumber},TopLayerNumber = {tempTopLayerNumber},Quantity = 0,BFinished = 0";
                    sqlServerExtension.UpdateRecord("PMS_Max_Export", content, $"where ExportName = '{WorkStationName}'");
                }
                string values = $"'{model.TASK_NO}','{model.PILE_NO}','{WorkStationName}','{model.METER_TYPE}',1,'{DateTime.Now.ToString()}'";
                sqlServerExtension.AddRecord("PMS_Max_MesPWTask", values);
                resultModel.RESULT_FLAG = "1";
            }
            sqlServerExtension.Disconnect();
            return Json(resultModel);
        }

        [AcceptVerbs("Get", "POST")]
        public JsonResult<ResultModel> setMeterCheckResult([FromBody] ExcerptWatchModel model)
        {
            ResultModel resultModel = new ResultModel();
            SQLServerExtension sqlServerExtension = new SQLServerExtension("server=.;uid=sa;pwd=123;database=PMS_KLDZ_6324");//192.168.91.249
            DataTable dataTable = sqlServerExtension.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveName = '{model.STATION_ID}'");
            if (dataTable == null || dataTable.Rows.Count == 0) { sqlServerExtension.Disconnect(); return Json(resultModel); }
            string WorkStation = dataTable.Rows[0]["WorkStationName"]?.ToString();
            int MaterialType = Convert.ToInt32(dataTable.Rows[0]["DeceiveType"]);
            List<string> TaskNoList = new List<string>();
            foreach (var item in model.METER)
            {
                if (!TaskNoList.Contains(item.TASK_NO)) { TaskNoList.Add(item.TASK_NO); }
                sqlServerExtension.UpdateRecord($"PMS_Max_{WorkStation}", $"TaskNo = '{item.TASK_NO}',Barcode = '{item.METER_CODE}',Quantity = '{(item.CHECK_RESULT == "0" ? "2" : "1")}',Status = 1",
                    $"where EpitopeNumber = '{item.METER_ID}'");
            }
            foreach (var str in TaskNoList)
            {
                sqlServerExtension.AddRecord("PMS_Max_MesEWTask", $"'{str}','0','{model.STATION_ID}',{MaterialType},1,1,'{DateTime.Now.ToString()}'");
                Thread.Sleep(1000);
                sqlServerExtension.AddRecord("PMS_Max_MesEWTask", $"'{str}','0','{model.STATION_ID}',{MaterialType},2,1,'{DateTime.Now.ToString()}'");
            }
            sqlServerExtension.Disconnect(); resultModel.RESULT_FLAG = "1"; return Json(resultModel);
        }

        [AcceptVerbs("Get", "POST")]
        public JsonResult<ResultModel> getStackAction([FromBody] MoveWatchModel model)
        {
           ResultModel resultModel = new ResultModel();
            SQLServerExtension sqlServerExtension = new SQLServerExtension("server=.;uid=sa;pwd=123;database=PMS_KLDZ_6324");
            DataTable dataTable = sqlServerExtension.QueryTable("PMS_Max_DeceiveManager", $"where DeceiveName in ('{model.START_PLATFORM_NO}','{model.DESTINATION_PLATFORM_NO}')");
            if (dataTable == null || dataTable.Rows.Count == 0) { sqlServerExtension.Disconnect(); return Json(resultModel); }
            string StartStation = "", EndStation = "";
            foreach (DataRow item in dataTable.Rows)
            {
                if (item["DeceiveName"]?.ToString() == model.START_PLATFORM_NO)
                {
                    StartStation = item["WorkStationName"]?.ToString();
                }
                else if (item["DeceiveName"]?.ToString() == model.DESTINATION_PLATFORM_NO)
                {
                    EndStation = item["WorkStationName"]?.ToString();
                }
            }
            DataTable dataTableMesMoveTask = sqlServerExtension.QueryTable("PMS_Max_MesMoveTask", $"where PileNo = '{model.PILE_NO}' and StartStation='{StartStation}' and TaskStatus=1");
            if (dataTableMesMoveTask == null || dataTableMesMoveTask.Rows.Count == 0)
            {
                string content = $"TaskNo = '0',PileNo = '{model.PILE_NO}',MaterialType = 0,LayerNumber = 6,TopLayerNumber = 0,Quantity = 0,BFinished = 1";//TaskNo = '0'
                sqlServerExtension.UpdateRecord("PMS_Max_Export", content, $"where ExportName = '{StartStation}'");
                sqlServerExtension.AddRecord("PMS_Max_MesMoveTask", $"'{model.PILE_NO}','{StartStation}','{EndStation}',1,'{DateTime.Now.ToString()}'");
                sqlServerExtension.Disconnect();
            }
            resultModel.RESULT_FLAG = "1"; 
            return Json(resultModel);
        }
    }
}
